# -*- coding: utf-8 -*-

from pkg_resources import (iter_entry_points, load_entry_point)
from ikazuchi.core.handler import NullHandler

try:
    from ikazuchi.locale import _
except ImportError:
    def _(s):
        return s

def get_plugin(opts):
    p = load_plugin(opts.plugin)
    translator = p.Translator if hasattr(p, "Translator") else None
    handler = p.Handler if hasattr(p, "Handler") else None
    try:
        handler_instance = handler(opts)
    except Exception as err:
        handler_instance = NullHandler()
        print _(u"Cannot instantiate Handler: {0}".format(err))
    return translator, handler_instance

def get_plugin_name(group="ikazuchi.plugins"):
    return [entry_point.name for entry_point in iter_entry_points(group)]

def get_plugin_distribution(name, group="ikazuchi.plugins"):
    for entry_point in iter_entry_points(group):
        if entry_point.name == name:
            return entry_point.dist

def load_plugin(name, group="ikazuchi.plugins"):
    dist = get_plugin_distribution(name)
    plugin = None
    try:
        plugin = dist.load_entry_point(group, name)
    except (ImportError, TypeError, ValueError) as err:
        print _(u"Load plugin error:")
        print _(u"  distribution: {0}".format(dist))
        print _(u"  group name  : {0}".format(group))
        print _(u"  plugin name : {0}".format(name))
        print _(u"  messages    : {0}".format(err))
    return plugin

def load_all_plugins(group="ikazuchi.plugins"):
    plugins = []
    for entry_point in iter_entry_points(group):
        try:
            plugins.append(entry_point.load())
        except ImportError as err:
            print "Load plugin error: {0}".format(err)
    return plugins



# -*- coding: utf-8 -*-

def add_linebreak(iterables):
    ret = []
    for i in iterables:
        if i:
            ret.append("{0}\n".format(i))
        else:
            ret.append(i)
    return ret

def del_linebreak(iterables):
    ret = []
    for i in iterables:
        if i:
            ret.append(i.replace("\n", ""))
        else:
            ret.append(i)
    return ret



# -*- coding: utf-8 -*-

import json
import urllib2
from contextlib import closing
from urllib import urlencode
from utils import get_ip_address

from ikazuchi.errors import NeedApiKeyError

__all__ = [
    "GoogleTranslator",
]

class GoogleTranslator(object):
    """
    Translator with Google Translate API with version 2
    http://code.google.com/intl/ja/apis/language/translate/overview.html
    http://code.google.com/intl/ja/apis/language/translate/v2/using_rest.html
    http://code.google.com/intl/ja/apis/language/translate/v2/pricing.html
    see also
    Google Translate API Terms of Use
    http://code.google.com/intl/ja/apis/language/translate/terms.html
    """
    domain = "www.googleapis.com"
    common_path = "/language/translate/"
    apikey = None  # get from configuration file
    q_format = "html"

    def __init__(self, lang_from, lang_to, handler):
        self.lang_from = lang_from
        self.lang_to = lang_to
        self.handler = handler
        self._url = "https://{0}{1}".format(self.domain, self.common_path)

    def get_url(self, func=None):
        if func:
            url = "{0}v2/{1}".format(self._url, func.func_name)
        else:
            url = "{0}v2".format(self._url)
        return url

    def request(self, url, data):
        req = urllib2.Request(url)
        req.add_header("X-HTTP-Method-Override", "GET")
        req.add_data(data)
        with closing(urllib2.urlopen(req)) as res:
            res_json = json.loads(res.read())
        return res_json["data"]

    def call_api(self, query, key, func=None):
        """high-level method to make url and request"""
        if not self.apikey:
            raise NeedApiKeyError("need API key !!!")
        url = self.get_url(func)
        response = self.request(url, urlencode(query, doseq=True))
        return self.api(), response[key]

    def detect(self, texts):
        """ Detect Language
        http://code.google.com/intl/ja/apis/language/translate/v2/\
        using_rest.html#detect-language
        """
        query = {
            "key": self.apikey,
            "q": [t.encode("utf-8") for t in texts],
        }
        return self.call_api(query, u"detections", func=self.detect)

    def languages(self):
        """ Discover Supported Languages
        http://code.google.com/intl/ja/apis/language/translate/v2/\
        using_rest.html#supported-languages
        """
        query = {
            "key": self.apikey,
            "target": self.lang_to,
        }
        return self.call_api(query, u"languages", func=self.languages)

    def translate(self, texts):
        """ Translate Text
        http://code.google.com/intl/ja/apis/language/translate/v2/\
        using_rest.html#Translate
        """
        query = {
            "format": self.q_format,
            "key": self.apikey,
            "q": [t.encode("utf-8") for t in texts],
            "source": self.lang_from,
            "target": self.lang_to,
        }
        api, response = self.call_api(query, u"translations")
        return api, [self.parse_html(r[u"translatedText"]) for r in response]

    def translate_tts(self, text, lang, f):
        """ Unofficial Google Text To Speech API
        http://weston.ruter.net/projects/google-tts/
        """
        headers = {
            "User-Agent": "Mozilla/5.0 "
                          "(Macintosh; U; Intel Mac OS X 10_6_7; en-US) "
                          "AppleWebKit/534.16 (KHTML, like Gecko) "
                          "Chrome/10.0.648.204 Safari/534.16"
        }
        query = {"tl": lang, "q": text.encode("utf-8")}
        url = "http://translate.google.com/translate_tts?{0}".format(
                urlencode(query))
        req = urllib2.Request(url, None, headers)
        with closing(urllib2.urlopen(req)) as res:
            f.write(res.read())
            f.flush()
        return self.api()



# -*- coding: utf-8 -*-

import threading

_MAX_SEMAPHORE = 10

def call_api_with_multithread(api_method, target_lines):
    def worker(line, results, i, semaphore):
        with semaphore:
            results[i] = api_method(line)

    results = [("", "")] * len(target_lines)
    s = threading.Semaphore(_MAX_SEMAPHORE)
    for i, line in enumerate(target_lines):
        if line:
            t = threading.Thread(target=worker, args=(line, results, i, s))
            t.start()
    # waiting for threads to complete
    main_thread = threading.currentThread()
    for t in threading.enumerate():
        if t is not main_thread:
            t.join()
    return results

def get_ip_address():
    import socket
    try:
        ip = socket.gethostbyname(socket.gethostname())
    except:
        ip = "127.0.0.1"
    return ip



# -*- coding: utf-8 -*-

import urllib2
from contextlib import closing
from urllib import urlencode
from xml.etree import ElementTree as ET

__all__ = [
    "MicrosoftTranslator"
]

class MicrosoftTranslator(object):
    """
    Microsoft Translator
    http://www.microsofttranslator.com/dev/
    see also
    Microsoft Terms of Use
    http://www.bing.com/developers/tou.aspx
    """
    domain = "api.microsofttranslator.com"
    common_path = "/V2/Http.svc/"
    apikey = "D9D0E326A70EA4E66218F43130890052808A0142"
    max_trans = "5"
    schema = {
        "base": "http://schemas.microsoft.com/2003/10/Serialization/",
        "array": "http://schemas.microsoft.com/2003/10/Serialization/Arrays",
        "mt": "http://schemas.datacontract.org/2004/07/"
                "Microsoft.MT.Web.Service.V2",
    }
    xml_attr = {
        "xmlns_array": {"xmlns": schema["array"]},
        "xmlns_mt": {"xmlns": schema["mt"]},
    }
    xml_tag = dict([(key, {
                            "int": "{%s}int" % schema[key],
                            "str": "{%s}string" % schema[key],
                          }) for key in schema.keys()])
    xml_translate_array_tag = [
        "{%s}ArrayOfTranslateArrayResponse" % schema["mt"],
        "{%s}TranslateArrayResponse" % schema["mt"],
        "{%s}From" % schema["mt"],
        "{%s}OriginalTextSentenceLengths" % schema["mt"],
        "{%s}int" % schema["array"],
        "{%s}TranslatedText" % schema["mt"],
        "{%s}TranslatedTextSentenceLengths" % schema["mt"],
        "{%s}int" % schema["array"],
    ]
    xml_type = {
        "int": ("ArrayOfint", "int"),
        "str": ("ArrayOfstring", "string"),
        "opt": ("TranslateOptions", None),
        "req": ("TranslateArrayRequest", None),
        "gettrans_req": ("GetTranslationsArrayRequest", None),
    }
    content_type = {
        "text": "text/plain",
        "html": "text/html",
        "xml": "text/xml; charset=utf-8",
    }

    def __init__(self, lang_from, lang_to, handler):
        self.lang_from = lang_from
        self.lang_to = lang_to
        self.handler = handler
        self._url = "http://{0}{1}".format(self.domain, self.common_path)

    def get_api_path(self, func):
        return "{0}?".format(func.func_name.title().replace("_", ""))

    def get_url(self, func, query):
        return "{0}{1}{2}".format(self._url,
                self.get_api_path(func), urlencode(query))

    def request(self, url, tag=None, post_data=None):
        items = []
        req = urllib2.Request(url)
        if post_data:
            req.add_header("Content-type", post_data["type"])
            req.add_data(post_data["data"])
        try:
            with closing(urllib2.urlopen(req)) as res:
                element = ET.parse(res)
                items = [i.text for i in element.getiterator(tag)]
        except urllib2.HTTPError as err:
            # FIXME: consider later
            err_msg = err.read()
            print err_msg
        return items

    def call_api(self, func, query, tag=None, post_data=None):
        """high-level method to make url and request"""
        url = self.get_url(func, query)
        items = self.request(url, tag, post_data)
        return self.api(), items

    def _set_tree(self, tree, items, attr):
        for key, value in items:
            tree.start(key, attr)
            tree.data(value)
            tree.end(key)

    def _set_tree_with_array(self, tree, items, data_type):
        for i in items:
            tree.start(data_type, self.xml_attr["xmlns_array"])
            tree.data(i)
            tree.end(data_type)

    def serialize_array(self, items, array_type, data_type):
        tree = ET.TreeBuilder()
        tree.start(array_type, self.xml_attr["xmlns_array"])
        self._set_tree_with_array(tree, items, data_type)
        tree.end(array_type)
        return ET.tostring(tree.close())

    def serialize_option(self, items, array_type):
        tree = ET.TreeBuilder()
        tree.start(array_type, self.xml_attr["xmlns_mt"])
        for key, value in items:
            tree.start(key, {})
            tree.data(value)
            tree.end(key)
        tree.end(array_type)
        return ET.tostring(tree.close())

    def serialize_translate_array(self, param, array_type):
        tree = ET.TreeBuilder()
        tree.start(array_type, {})
        for key, value in param:
            tree.start(key, {})
            if key == "Options":
                self._set_tree(tree, value, self.xml_attr["xmlns_mt"])
            elif key == "Texts":
                self._set_tree_with_array(tree, value, "string")
            else:
                tree.data(value)
            tree.end(key)
        tree.end(array_type)
        return ET.tostring(tree.close())

    def break_sentences(self, text):
        """ BreakSentences Method
        http://msdn.microsoft.com/en-us/library/ff512410.aspx
        """
        query = {
            "appId": self.apikey,
            "text": text.encode("utf-8"),
            "language": self.lang_from,
        }
        return self.call_api(self.break_sentences, query,
                             self.xml_tag["array"]["int"])

    def detect(self, text):
        """ Detect Method
        http://msdn.microsoft.com/en-us/library/ff512411.aspx
        """
        query = {"appId": self.apikey, "text": text.encode("utf-8")}
        api, items = self.call_api(self.detect, query,
                                   self.xml_tag["base"]["str"])
        return api, items[0]

    def detect_array(self, texts):
        """ DetectArray Method
        http://msdn.microsoft.com/en-us/library/ff512412.aspx
        """
        query = {"appId": self.apikey}
        data = {
            "type": self.content_type["xml"],
            "data": self.serialize_array(texts, *self.xml_type["str"]),
        }
        return self.call_api(self.detect_array, query,
                             self.xml_tag["array"]["str"], data)

    def get_language_names(self, lang_codes):
        """ GetLanguageNames Method
        http://msdn.microsoft.com/en-us/library/ff512414.aspx
        """
        query = {"appId": self.apikey, "locale": self.lang_to}
        data = {
            "type": self.content_type["xml"],
            "data": self.serialize_array(lang_codes, *self.xml_type["str"]),
        }
        return self.call_api(self.get_language_names, query,
                             self.xml_tag["array"]["str"], data)

    def get_languages_for_speak(self):
        """ GetLanguagesForSpeak Method
        http://msdn.microsoft.com/en-us/library/ff512415.aspx
        """
        query = {"appId": self.apikey}
        return self.call_api(self.get_languages_for_speak, query,
                             self.xml_tag["array"]["str"])

    def get_languages_for_translate(self):
        """ GetLanguagesForTranslate Method
        http://msdn.microsoft.com/en-us/library/ff512416.aspx
        """
        query = {"appId": self.apikey}
        return self.call_api(self.get_languages_for_translate, query,
                             self.xml_tag["array"]["str"])

    def get_translations(self, text):
        """ GetTranslations Method
        http://msdn.microsoft.com/en-us/library/ff512417.aspx
        """
        query = {
            "appId": self.apikey,
            "text": text.encode("utf-8"),
            "from": self.lang_from,
            "to": self.lang_to,
            "maxTranslations": self.max_trans,
        }
        options = [
            ("Category", "general"),
            ("ContentType", self.content_type["html"]),
            ("Uri", ""),
            ("User", ""),
            ("State", ""),
        ]
        data = {
            "type": self.content_type["xml"],
            "data": self.serialize_option(options, self.xml_type["opt"][0])
        }
        return self.call_api(self.get_translations, query, None, data)

    def get_translations_array(self, texts):
        """ GetTranslationsArray Method
        http://msdn.microsoft.com/en-us/library/ff512418.aspx
        """
        query = {"appId": self.apikey}
        params = [
            ("AppId", self.apikey),
            ("From", self.lang_from),
            ("Options", [
                ("Category", "general"),
                ("ContentType", self.content_type["html"]),
                ("Uri", ""),
                ("User", ""),
                ("State", ""),
            ]),
            ("Texts", texts),
            ("To", self.lang_to),
            ("MaxTranslations", self.max_trans),
        ]
        data = {
            "type": self.content_type["xml"],
            "data": self.serialize_translate_array(
                        params, self.xml_type["gettrans_req"][0]),
        }
        return self.call_api(self.get_translations_array, query, None, data)

    def speak(self, text, lang, f):
        """ Speak Method
        http://msdn.microsoft.com/en-us/library/ff512420.aspx
        """
        # FIXME: consider later
        query = {
            "appId": self.apikey,
            "text": text.encode("utf-8"),
            "language": lang,
            "format": "audio/wav",
        }
        url = self.get_url(self.speak, query)
        req = urllib2.Request(url)
        with closing(urllib2.urlopen(req)) as res:
            f.write(res.read())
            f.flush()
        return self.api()

    def translate(self, text):
        """ Translate Method
        http://msdn.microsoft.com/en-us/library/ff512421.aspx
        """
        query = {
            "appId": self.apikey,
            "text": text.encode("utf-8"),
            "from": self.lang_from,
            "to": self.lang_to,
            "contentType": self.content_type["html"],
            "category": "general",
        }
        api, items = self.call_api(self.translate, query,
                                   self.xml_tag["base"]["str"])
        return api, self.parse_html(items[0])

    def translate_array(self, texts):
        """ TranslateArray Method
        http://msdn.microsoft.com/en-us/library/ff512422.aspx
        """
        query = {"appId": self.apikey}
        params = [
            ("AppId", self.apikey),
            ("From", self.lang_from),
            ("Options", [
                ("Category", "general"),
                ("ContentType", self.content_type["html"]),
                ("Uri", ""),
                ("User", ""),
                ("State", ""),
            ]),
            ("Texts", texts),
            ("To", self.lang_to),
        ]
        data = {
            "type": self.content_type["xml"],
            "data": self.serialize_translate_array(
                        params, self.xml_type["req"][0]),
        }
        api, items = self.call_api(self.translate_array, query, None, data)
        step = 7
        trans = [self.parse_html(items[i]) for i in
                        xrange(5, len(texts) * step, step)]
        return api, trans



# -*- coding: utf-8 -*-

import argparse
import os
import sys

from core.handler import *
from locale import _

def get_lang():
    from locale import getdefaultlocale
    lang, country = getdefaultlocale()[0].split("_")
    return lang

def get_handler(opts):
    h = NullHandler()
    if opts.lang:
        h = LanguageHandler(opts.api, opts.encoding)
    elif opts.sentences:
        h = SingleSentenceHandler(opts)
    return h

class EncodingAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        enc = "".join(values.split()).split(",")
        enc_out = enc[1:] and enc[1] or enc[0]
        setattr(namespace, self.dest, [enc[0], enc_out])

def check_encoding(encoding):
    ret = []
    for enc in encoding:
        try:
            unicode("test", enc)
        except LookupError:
            ret.append(enc)
    return ret

def set_default_encoding(opts):
    from locale import getdefaultlocale
    enc = getdefaultlocale()[1]
    opts.encoding = [enc, enc]

def convert_str_to_unicode(opts):
    if opts.sentences:
        opts.sentences = [unicode(s, opts.encoding[0]) for s in opts.sentences]

def get_command(cmd, var="PATH"):
    import platform
    os_name = platform.system()
    delim = ";" if os_name == "Windows" else ":"
    # like a which command
    for path in os.getenv(var).split(delim):
        path_cmd = os.path.join(path, cmd)
        if os.access(path_cmd, os.X_OK):
            yield path_cmd

_UNSUPPORTED_VERSION = _("Unsupported Python version, use 2.6 above")

def check_python_version():
    if sys.version_info < (2, 6):
        print _UNSUPPORTED_VERSION
        sys.exit(0)



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        [u".. seealso::",
         u"",
         u"    Module :mod:`ikazuchi`",
         u"        ikazuchi is a tool",
         u"        to call api.",
         u"        it helps translation.",
         u"",
         u"    :ref:`ikazuchi-samples`",
         u"        ikazuchi has lots of samples.",
         u""],
        u".. seealso::",

        [u".. seealso::",
         "\n",
         u"    Module <span class=notranslate>:mod:`ikazuchi`</span>\n",
         u"        ikazuchi is a tool to call api. it helps translation.\n",
         "\n",
         u"    <span class=notranslate>:ref:`ikazuchi-samples`</span>\n",
         u"        ikazuchi has lots of samples.\n",
         "\n"]
    ),
    # (
    #     [u".. seealso::",
    #      u"",
    #      u"    :ref:`ikazuchi-samples`",
    #      u"        ikazuchi has lots of samples.",
    #      u"",
    #      u"          - command line use",
    #      u"          - po file use",
    #      u"          - rst file use",
    #      u""],
    #
    #     [u".. seealso::",
    #      u"",
    #      u"    <span class=notranslate>:ref:`ikazuchi-samples`</span>\n",
    #      u"        ikazuchi has lots of samples.\n",
    #      u"",
    #      u"          - command line use\n",
    #      u"          - po file use\n",
    #      u"          - rst file use\n",
    #      u""]
    # ),
    # (
    #     [u".. seealso::",
    #      u"",
    #      u"    :ref:`ikazuchi-samples`",
    #      u"        ikazuchi has lots of samples.",
    #      u"",
    #      u"          | command line use",
    #      u"          |   only translate",
    #      u"          |   translate with audio",
    #      u""],
    #
    #     [u".. seealso::",
    #      u"",
    #      u"    <span class=notranslate>:ref:`ikazuchi-samples`</span>\n",
    #      u"        ikazuchi has lots of samples.\n",
    #      u"",
    #      u"          | command line use\n",
    #      u"          |   only translate\n",
    #      u"          |   translate with audio\n",
    #      u""]
    # ),
]



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        [u"* first line",
         u""],

        [u"* first line\n",
         u"\n"]
    ),
    (
        [u"* first line"],

        [u"* first line\n"]
    ),
    (
        [u"* first line",
         u"  sub line"],

        [u"* first line sub line\n"]
    ),
    (
        [u"* first line",
         u"* second line",
         u""],

        [u"* first line\n",
         u"* second line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"* second line"],

        [u"* first line\n",
         u"* second line\n"]
    ),
    (
        [u"* first line",
         u"  * second line",
         u""],

        [u"* first line\n",
         u"  * second line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  * second line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  * second line",
         u"    * third line",
         u""],

        [u"* first line\n",
         u"  * second line\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  sub line",
         u"  * second line",
         u"    * third line",
         u""],

        [u"* first line sub line\n",
         u"  * second line\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  * second line",
         u"    sub line",
         u"    * third line",
         u""],

        [u"* first line\n",
         u"  * second line sub line\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  * second line",
         u"    * third line",
         u"      sub line",
         u""],

        [u"* first line\n",
         u"  * second line\n",
         u"    * third line sub line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  sub line",
         u"  and then",
         u"  * second line",
         u"    * third line",
         u""],

        [u"* first line sub line and then\n",
         u"  * second line\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  * second line",
         u"    sub line",
         u"    and then",
         u"    * third line",
         u""],

        [u"* first line\n",
         u"  * second line sub line and then\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  * second line",
         u"    * third line",
         u"      sub line",
         u"      and then",
         u""],

        [u"* first line\n",
         u"  * second line\n",
         u"    * third line sub line and then\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"",
         u"    * third line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  * second line\n",
         u"\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  sub line",
         u"",
         u"  * second line",
         u"",
         u"    * third line",
         u""],

        [u"* first line sub line\n",
         u"\n",
         u"  * second line\n",
         u"\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"    sub line",
         u"",
         u"    * third line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  * second line sub line\n",
         u"\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"",
         u"    * third line",
         u"      sub line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  * second line\n",
         u"\n",
         u"    * third line sub line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"  sub line",
         u"  and then",
         u"",
         u"  * second line",
         u"",
         u"    * third line",
         u""],

        [u"* first line sub line and then\n",
         u"\n",
         u"  * second line\n",
         u"\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"    sub line",
         u"    and then",
         u"",
         u"    * third line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  * second line sub line and then\n",
         u"\n",
         u"    * third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"",
         u"    * third line",
         u"      sub line",
         u"      and then",
         u""],

        [u"* first line\n",
         u"\n",
         u"  * second line\n",
         u"\n",
         u"    * third line sub line and then\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"  - third line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  - second line\n",
         u"  - third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    - third line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  - second line\n",
         u"\n",
         u"    - third line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    * third line",
         u"",
         u"  - fourth line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  - second line\n",
         u"\n",
         u"    * third line\n",
         u"\n",
         u"  - fourth line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    * third line",
         u"      sub line",
         u"",
         u"  - fourth line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  - second line\n",
         u"\n",
         u"    * third line sub line\n",
         u"\n",
         u"  - fourth line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    * third line",
         u"      sub line",
         u"  - fourth line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  - second line\n",
         u"\n",
         u"    * third line sub line\n",
         u"  - fourth line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    * third line",
         u"    * fourth line",
         u"",
         u"  - fifth line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  - second line\n",
         u"\n",
         u"    * third line\n",
         u"    * fourth line\n",
         u"\n",
         u"  - fifth line\n",
         u"\n"]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    * third line",
         u"      sub line",
         u"    * fourth line",
         u"      sub line",
         u"",
         u"  - fifth line",
         u""],

        [u"* first line\n",
         u"\n",
         u"  - second line\n",
         u"\n",
         u"    * third line sub line\n",
         u"    * fourth line sub line\n",
         u"\n",
         u"  - fifth line\n",
         u"\n"]
    ),
]



# -*- coding: utf-8 -*-

DATA_SET = [
    (   # no indent line
         u"no indent line.",

        [u"no indent line.\n"],
    ),
    (   # indent line
         u"  indent line.",

        [u"  indent line.\n"],
    ),
    (   # no indent and 1 sentece and others
         u"no indent line。next line",

        [u"no indent line。\n", u"next line\n"]
    ),
    (   # no indent and 2 senteces
         u"no indent line。next line。",

        [u"no indent line。\n", u"next line。\n"]
    ),
    (   # indent and 2 senteces
         u"  indent line。next line。",

        [u"  indent line。\n", u"  next line。\n"]
    ),
    (   # indent and 2 senteces, 2nd line has leading space
         u"  indent line。 next line。",

        [u"  indent line。\n", u"  next line。\n"]
    ),
    (   # complex lines
         u"  indent line。 next line。third line？ forth line！",

        [u"  indent line。\n",
         u"  next line。\n",
         u"  third line？\n",
         u"  forth line！\n"]
    ),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys

from conf import (get_conf, get_conf_path)
from core.translator import TRANSLATE_API
from errors import *
from locale import _
from plugins.utils import (get_plugin, load_all_plugins)
from utils import *

__version__ = "0.5.6"

# base parser object for common option
base_parser = argparse.ArgumentParser(add_help=False)
base_parser.set_defaults(api=None, lang_from="en", lang_to=get_lang(),
                         encoding=None, quiet=False, verbose=False)
base_parser.add_argument("-a", "--api", dest="api", metavar="API",
                         help=u"APIs are {0}".format(TRANSLATE_API.keys()))
base_parser.add_argument("-e", "--encoding", dest="encoding",
                         action=EncodingAction, metavar="ENCODING",
                         help=u"input/output encoding")
base_parser.add_argument("-f", "--from", dest="lang_from", metavar="LANG",
                         help=u"original language")
base_parser.add_argument("-q", "--quiet", dest="quiet", action="store_true",
                         help=u"not to show original sentence to stdout")
base_parser.add_argument("-t", "--to", dest="lang_to", metavar="LANG",
                         help=u"target language to translate")

# actual object by using ikazuchi command
parser = argparse.ArgumentParser()
subparsers = parser.add_subparsers(
    dest="plugin",
    help=u"available plugins. 'normal' means ikazuchi's standard feature "
         u"so it can be abbreviated",
)

def get_args():
    normal_parser = subparsers.add_parser("normal", parents=[base_parser])
    normal_parser.set_defaults(detect=False, lang=False, sentences=[])
    normal_parser.add_argument("-d", "--detect", dest="detect",
        action="store_true", help=u"detect language for target sentence")
    normal_parser.add_argument("-l", "--languages", dest="lang",
        action="store_true", help=u"show supported languages")
    normal_parser.add_argument("-s", "--sentences", dest="sentences",
        nargs="+", metavar="SENTENCE", help=u"target sentences")
    normal_parser.add_argument("--version", action="version",
        version="%(prog)s {0}".format(__version__))

    # load subparsers from all plugins
    load_all_plugins()

    # little trick to abbreviate 'normal' keyword as default
    subparser_names = subparsers.choices.keys() + ["-h"]
    if not (len(sys.argv) >= 2 and sys.argv[1] in subparser_names):
        sys.argv[1:] = ["normal"] + sys.argv[1:]
    opts = parser.parse_args()

    if not opts.lang_to:
        opts.lang_to = raw_input(u"Type language code: ")

    err_msg = None
    if opts.encoding:
        err_encoding = check_encoding(opts.encoding)
        if err_encoding:
            err_msg = _(u"Unknown encodings: {0}").format(err_encoding)
    else:
        set_default_encoding(opts)

    # error check
    if not err_msg and opts.plugin == "normal":
        convert_str_to_unicode(opts)
        if not (opts.lang or opts.sentences):
            err_msg = _(u"Need to specify optional arguments")

    if err_msg:
        print err_msg
        sys.exit(0)

    return opts

def main():
    # pre process
    check_python_version()
    conf = get_conf(get_conf_path())

    # main process
    opts = get_args()
    if opts.plugin == "normal":
        handler = get_handler(opts)
    else:
        plugin_translator, handler = get_plugin(opts)
        if plugin_translator:
            TRANSLATE_API[opts.plugin] = plugin_translator
            opts.api = opts.api if opts.api else opts.plugin

    try:
        t = TRANSLATE_API[opts.api](opts.lang_from, opts.lang_to, handler)
        t.set_parameter_from_conf(conf)
        t.call_method_with_handler()
    except IkazuchiError as err:
        print err

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import os
import re
from os.path import (dirname, realpath)

try:
    from ikazuchi.locale import _
except ImportError:
    def _(s):
        return s

def get_and_check_file_access(f, mode="r"):
    """ check access permission and return the file
    >>> import tempfile
    >>> f = tempfile.NamedTemporaryFile()
    >>> file_name = get_and_check_file_access(f.name)
    >>> f.name == file_name
    True
    >>> file_name = get_and_check_file_access(f.name, "w")
    >>> f.name == file_name
    True
    """
    if mode == "r":
        if not os.access(f, os.R_OK):
            raise ValueError(_(u"Cannot access file: {0}").format(f))
    elif mode == "w":
        if not os.access(realpath(dirname(f)), os.W_OK):
            raise ValueError(_(u"Cannot write file: {0}").format(f))
    return f

def concatenate_lines(lines, pattern):
    """ concatenate lines
    >>> import re
    >>> pattern = re.compile(r"(^\s+)(.+?)")
    >>> lines = [u"a", u" b", u" c", u" d", u"e"]
    >>> concatenate_lines(lines, pattern)
    [u'a', u' b c d', u'e']
    >>> lines = [u"a", u" b", u"  c", u"  d", u"e"]
    >>> concatenate_lines(lines, pattern)
    [u'a', u' b', u'  c d', u'e']
    >>> lines = [u"a", u" b", u"  c", u" d", u"e"]
    >>> concatenate_lines(lines, pattern)
    [u'a', u' b', u'  c', u' d', u'e']
    >>> lines = [u"a", u" b", u"c", u" d", u" e"]
    >>> concatenate_lines(lines, pattern)
    [u'a', u' b', u'c', u' d e']
    >>> lines = [u"a", u" b", u" c", u" d", u"  e"]
    >>> concatenate_lines(lines, pattern)
    [u'a', u' b c d', u'  e']
    >>> pattern = re.compile(r"(^\s*)(.+?)")
    >>> lines = [u"a", u"b", u"c", u" d", u"e"]
    >>> concatenate_lines(lines, pattern)
    [u'a b c', u' d', u'e']
    """
    _lines, prev_prefix = [], None
    for line in lines:
        match = re.search(pattern, line)
        if match:
            prefix, text = match.groups()
            if prev_prefix == prefix:
                _lines[-1] = u"{0} {1}".format(_lines[-1].rstrip(), text)
            else:
                _lines.append(line)
            prev_prefix = prefix
        else:
            _lines.append(line)
            prev_prefix = None
    return _lines

def get_multiline(lines, range_num):
    """ read lines every range_num
    >>> list(get_multiline(["a", "b", "c"], 2))
    [['a', 'b'], ['b', 'c']]
    >>> list(get_multiline(["a", "b", "c", "d"], 2))
    [['a', 'b'], ['b', 'c'], ['c', 'd']]
    >>> list(get_multiline(["a", "b", "c"], 3))
    [['a', 'b', 'c']]
    >>> list(get_multiline(["a", "b", "c", "d"], 3))
    [['a', 'b', 'c'], ['b', 'c', 'd']]
    >>> list(get_multiline(["a", "b"], 3))
    [['a', 'b']]
    >>> list(get_multiline(["a", "b"], 5))
    [['a', 'b']]
    >>> list(get_multiline([], 2))
    [[]]
    """
    to = range_num * (-1) + 1
    if len(lines) <= abs(to):
        yield lines
    else:
        for num, _ in enumerate(lines[:to]):
            yield lines[num:num + range_num]

def get_sequential_block(lines, compare_func):
    """
    >>> import re
    >>> pattern = re.compile(r"\w+")
    >>> cmp_func = lambda s: not re.search(pattern, s)
    >>> get_sequential_block([], cmp_func)
    (0, [])
    >>> get_sequential_block(["a", "b", "\\n"], cmp_func)
    (1, ['a', 'b'])
    >>> get_sequential_block(["a", "b", "c", "\\n"], cmp_func)
    (2, ['a', 'b', 'c'])
    >>> pattern = re.compile(r"\\n")
    >>> cmp_func = lambda s: re.search(pattern, s)
    >>> get_sequential_block(["a", "b", "c", "\\n"], cmp_func)
    (2, ['a', 'b', 'c'])
    """
    num = 0
    if len(lines) >= 2:
        for num, mline in enumerate(get_multiline(lines, 2)):
            if compare_func(mline[1]):
                break
        else:
            # maybe read out to EOF
            num += 1
    return num, lines[0:num + 1]

def get_east_asian_width(unicode_str):
    """
    Na: Narrow    1 半角英数
    H : Halfwidth 1 半角カナ
    W : Wide      2 全角文字
    F : Fullwidth 2 全角英数
    N : Neutral   1 アラビア文字
    A : Ambiguous 1 ギリシア文字、キリル文字(locale で幅が違う)
    >>> get_east_asian_width(unicode("1２3あいうabc", "utf-8"))
    13
    >>> get_east_asian_width(unicode("ﾊﾝｶｸｶﾅはどうかな？", "utf-8"))
    18
    >>> get_east_asian_width(unicode("スペース   全角　　　", "utf-8"))
    21
    >>> get_east_asian_width("string")
    6
    """
    from unicodedata import east_asian_width
    if type(unicode_str) is not unicode:
        unicode_str = unicode(unicode_str, "utf-8")
    width = 0
    for i in unicode_str:
        width += 1
        if east_asian_width(i) in ("W", "F"):
            width += 1
    return width

def zip_with_flatlist(list1, list2):
    """
    >>> zip_with_flatlist(range(3), range(3,6))
    [0, 3, 1, 4, 2, 5]
    >>> zip_with_flatlist(range(1), range(2,5))
    [0, 2]
    """
    _list = []
    for i, j in zip(list1, list2):
        _list.append(i)
        _list.append(j)
    return _list



# -*- coding: utf-8 -*-

from base import BaseHandler

class SingleSentenceHandler(BaseHandler):
    """
    Handler class for translating single sentence
    """
    def __init__(self, opts):
        self.sentences = opts.sentences
        self.encoding = opts.encoding
        self.quiet = opts.quiet
        if not opts.api or opts.api == "microsoft":
            self.method_name = "translate_array"
        if opts.detect:
            self.method_name = "detect"
            if not opts.api or opts.api == "microsoft":
                self.method_name = "detect_array"

    def _encode(self, text):
        return text.encode(self.encoding[1])

    def _call_method(self, api_method):
        api, result = api_method(self.sentences)
        _method = self.method_name.replace("_array", "")  # for microsoft
        _method = u"{0}({1}):".format(_method, api)
        for num, s in enumerate(self.sentences):
            if not self.quiet:
                print self._encode(u"{0:25}{1}".format("sentence:", s))
            print self._encode(u"{0:25}{1}".format(_method, result[num]))



# -*- coding: utf-8 -*-

import gettext
from locale import *
from os.path import (dirname, realpath)

LOCALE_DIR = dirname(realpath(__file__))

t = gettext.translation("ikazuchi", localedir=LOCALE_DIR, fallback=True)
_ = t.ugettext



# -*- coding: utf-8 -*-

import sys
from nose.tools import *
from StringIO import StringIO

# functions for test
from ikazuchi.core.handler import *

class TestSingleSentenceHandler(object):

    class Option(object):
        api = "google"
        sentences = [""]
        encoding = ["utf-8", "utf-8"]
        quiet = False
        detect = False

    def setup(self):
        sys.stdout = StringIO()
        self.opts = TestSingleSentenceHandler.Option()

    def _dummy_translate(self, sentence):
        return "", sentence

    def test_with_quiet_option(self):
        self.opts.quiet = True
        h = SingleSentenceHandler(self.opts)
        h._call_method(self._dummy_translate)
        first_line = sys.stdout.getvalue().split('\n')[0]
        assert_equal("translate():             ", first_line)

    def test_detect_with_quiet_option(self):
        self.opts.quiet = True
        self.opts.detect = True
        h = SingleSentenceHandler(self.opts)
        h._call_method(self._dummy_translate)
        first_line = sys.stdout.getvalue().split('\n')[0]
        assert_equal("detect():                ", first_line)

    def test_without_quiet_option(self):
        h = SingleSentenceHandler(self.opts)
        h._call_method(self._dummy_translate)
        first_line = sys.stdout.getvalue().split('\n')[0]
        assert_equal("sentence:                ", first_line)



# -*- coding: utf-8 -*-

DATA_SET = [
    # standalone sourceblock
    (  # 0
        [u"::\n"
         u"",
         u"    first code",
         u"    second code",
         u""],
        u"::\n",

        [u"::\n",
         u"    first code",
         u"    second code",
         u""]
    ),
    (  # 1
        [u"::\n"
         u"",
         u"    first code",
         u"    second code"],
        u"::\n",

        [u"::\n",
         u"    first code",
         u"    second code"]
    ),
    (  # 2
        [u"::\n"
         u"",
         u"",
         u"    first code",
         u"    second code"],
        u"::\n",

        [u"::\n"
         u"",
         u"",
         u"    first code",
         u"    second code"]
    ),
    (  # 3
        [u"::  \n"
         u"",
         u"    first code",
         u"    second code",
         u""],
        u"::  \n",

        [u"::  \n",
         u"    first code",
         u"    second code",
         u""]
    ),
    (  # 4
        [u"::\n"
         u"    first code",
         u"    second code",
         u""],
        u"::\n",

        [u"::\n    first code",
         u"    second code",
         u""]
    ),

    # given first line has "::" and test
    (  # 5
        [u"that is::\n"
         u"",
         u"    first code",
         u"    second code",
         u""],
        u"that is::\n",

        [u"that is<span class=notranslate>::</span>\n",
         u"    first code",
         u"    second code",
         u""]
    ),
    (  # 6
        [u"that is::\n"
         u"",
         u"    first code",
         u"    second code"],
        u"that is::\n",

        [u"that is<span class=notranslate>::</span>\n",
         u"    first code",
         u"    second code"]
    ),
    (  # 7
        [u"that is::\n"
         u"",
         u"",
         u"    first code",
         u"    second code"],
        u"that is::\n",

        [u"that is<span class=notranslate>::</span>\n",
         u"",
         u"    first code",
         u"    second code"]
    ),
    (  # 8
        [u"that is::  \n"
         u"",
         u"    first code",
         u"    second code",
         u""],
        u"that is::  \n",

        [u"that is<span class=notranslate>::  </span>\n",
         u"    first code",
         u"    second code",
         u""]
    ),
    (  # 9
        [u"that is::\n",
         u"    first code",
         u"    second code",
         u""],
        u"that is::\n",

        [u"that is<span class=notranslate>::</span>\n",
         u"    first code",
         u"    second code",
         u""]
    ),
]



# -*- coding: utf-8 -*-

import ConfigParser
import os
from os.path import join as pathjoin

try:
    from ikazuchi.locale import _
except ImportError:
    def _(s):
        return s

CONF_DIR = u".ikazuchi"
CONF_FILE = u"ikazuchi.conf"
CONF_PATH = pathjoin(CONF_DIR, CONF_FILE)
SECTIONS = [
    ("general", ["http_proxy", "https_proxy"]),
    ("google", ["apikey"]),
    ("microsoft", ["apikey"]),
]

GOOGLE_APIKEY_REGISTER = "https://code.google.com/apis/console/"
MICROSOFT_APIKEY_REGISTER = "http://www.bing.com/developers"

_UNIX_LIKE_SYSTEMS = ("Darwin", "Linux", "FreeBSD")


def get_conf_path():
    """
    >>> get_conf_path().rsplit("/", 2)[-2:]
    [u'.ikazuchi', u'ikazuchi.conf']
    """
    import platform
    _os = platform.system()
    if _os in _UNIX_LIKE_SYSTEMS:
        prefix = os.getenv(u"HOME")
    elif _os == "Windows":
        prefix = os.getenv(u"APPDATA")
    return pathjoin(prefix, CONF_PATH)

def get_conf(conf_file):
    conf = ConfigParser.SafeConfigParser()
    has_conf_file = True
    if not os.access(conf_file, os.R_OK):
        create_conf_file(conf_file)
        has_conf_file = False
    conf.read(conf_file)
    get_or_set_sections(conf)
    # write conf file as template
    if not has_conf_file:
        with open(conf_file, "wb") as f:
            conf.write(f)
    return conf

def get_or_set_sections(conf):
    for section, options in SECTIONS:
        if not conf.has_section(section):
            conf.add_section(section)
        get_or_set_options(conf, section, options)

def get_or_set_options(conf, section, options):
    for option in options:
        try:
            conf.get(section, option)
        except ConfigParser.NoOptionError:
            conf.set(section, option, "")

def create_conf_file(conf_file):
    conf_dir = conf_file.rsplit("/", 1)[0]
    make_dirs(conf_dir)
    with open(conf_file, "wb") as f:
        pass  # menas touch command
    print _(u"created conf file: {0}".format(conf_file))

def make_dirs(path):
    if not os.access(path, os.F_OK):
        os.makedirs(path)
        print _(u"made directory: {0}".format(path))

def show_how_to_get_apikey():
    print _(u"You should get API KEY from each Translate API")
    print _(u"Google: {0}".format(GOOGLE_APIKEY_REGISTER))
    print _(u"Microsoft: {0}".format(MICROSOFT_APIKEY_REGISTER))
    print _(u"Set apikey = xxx in {0}".format(get_conf_path()))
    print _(u"")



# -*- coding: utf-8 -*-

import ConfigParser
import os
from nose.tools import *
from os.path import (dirname, join as pathjoin)

# functions for test
from ikazuchi.conf import *

def _assert(expected, actual):
    assert_equal(expected, actual)

class TestExistingConfigFile(object):

    conf_file = pathjoin(dirname(__file__), "data/conf/ikazuchi.conf")

    def test_get_conf(self):
        expected = {
            "general": {"http_proxy": "localhost:8080",
                        "https_proxy": "localhost:8080"},
            "google": {"apikey": "xxx"},
            "microsoft": {"apikey": "yyy"},
        }
        conf = get_conf(self.conf_file)
        for section, options in SECTIONS:
            for opt in options:
                yield _assert, expected[section][opt], conf.get(section, opt)


class TestNotExistingConfigFile(object):

    conf_file = pathjoin(dirname(__file__), "data/conf/notexisting.conf")

    def setup(self):
        self._remove_conf()

    def teardown(self):
        self._remove_conf()

    def _remove_conf(self):
        if os.access(self.conf_file, os.R_OK):
            os.remove(self.conf_file)

    def test_get_conf(self):
        expected = {
            "general": {"http_proxy": "", "https_proxy": ""},
            "google": {"apikey": ""},
            "microsoft": {"apikey": ""},
        }
        conf = get_conf(self.conf_file)
        for section, options in SECTIONS:
            for opt in options:
                yield _assert, expected[section][opt], conf.get(section, opt)

    def test_create_conf_file(self):
        yield _assert, False, os.access(self.conf_file, os.R_OK)
        create_conf_file(self.conf_file)
        yield _assert, True, os.access(self.conf_file, os.R_OK)

    def test_get_or_set_sections(self):
        conf = ConfigParser.SafeConfigParser()
        get_or_set_sections(conf)
        _assert([s[0] for s in SECTIONS], conf.sections())

    def test_get_or_set_options(self):
        conf = ConfigParser.SafeConfigParser()
        expected = {
            "general": [("http_proxy", ""), ("https_proxy", "")],
            "google": [("apikey", "")],
            "microsoft": [("apikey", "")],
        }
        get_or_set_sections(conf)
        for section, options in SECTIONS:
            get_or_set_options(conf, section, options)
            yield _assert, expected[section], conf.items(section)



# -*- coding: utf-8 -*-

try:
    import vim
except ImportError:
    print "call ':pyfile raimei' from vim"

from ikazuchi.core.translator import TRANSLATE_API
from ikazuchi.plugins.rstfile import reSTParser, reSTApiCaller
from ikazuchi.vim.utils import (to_encode, get_vim_variables,
                                get_apikey, get_translate_method)
from os.path import splitext

def get_target_lines(start, end, enc):
    _lines = vim.current.buffer[start:end]
    return [unicode("{0}\n".format(line), enc) for line in _lines]

def get_target_range():
    # take a range until next blank line if target range is not specified
    cur_row, cur_column = vim.current.window.cursor
    start = vim.current.range.start
    end = vim.current.range.end + 1
    if cur_row == end and (start + 1) == end:
        max_row = len(vim.current.buffer)
        while vim.current.buffer[end:end + 1] != [""] and end < max_row:
            end += 1
    return start, end

def translate_with_range(api_name, api_method, enc, lang_to):
    start, end = get_target_range()
    target_lines = get_target_lines(start, end, enc)
    caller = reSTApiCaller(reSTParser(target_lines).parse(), lang_to)
    trans = [l.encode(enc) for lines in caller.call(api_method) for l in lines]
    # previous and last empty lines are just for look and feel
    translated = ["", ] + trans + ["", ]
    # add translated text into vim
    vim.current.buffer.append(translated, end)
    vim.command("let raimei_target_lines={0}".format(
                to_encode(target_lines, enc)))
    print "Translated by {0}".format(api_name.title())

def translate(api_name, lang_from, lang_to, enc):
    translator = TRANSLATE_API[api_name](lang_from, lang_to, None)
    translator.set_apikey(get_apikey(api_name))
    api_method = get_translate_method(translator, api_name)
    translate_with_range(api_name, api_method, enc, lang_to)

def comment_out_original_lines():
    start, end = get_target_range()
    start += 1
    vim.current.buffer.append("..", start - 1)
    for num, line in enumerate(vim.current.buffer[start:end + 1]):
        vim.current.buffer[start + num] = "    {0}".format(line)

def main():
    try:
        vim_vars = get_vim_variables()
        translate(*vim_vars)
        ext = splitext(vim.current.buffer.name)[1][1:]
        if ext == "rst":
            comment_out_original_lines()
    except Exception as err:
        print "Got error: {0}".format(err)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

DATA_SET = [
    # normal paragraph
    (
        [u"first line",
         u"second line",
         u""],
        [("p",
          [u"first line",
           u"second line"],
          u""),
         1]
    ),
    (
        [u"first line",
         u"second line"],
        [("p",
          [u"first line",
           u"second line"],
          u""),
         1]
    ),
    (
        [u"first line"],
        [("p",
          [u"first line"],
          u""),
         0]
    ),
    (
        [u"first line",
         u"  second line"],
        [("p",
          [u"first line",
           u"  second line"],
          u""),
         1]
    ),
    (
        [u"first line",
         u"  second line",
         u"    third line"],
        [("p",
          [u"first line",
           u"  second line",
           u"    third line"],
          u""),
         2]
    ),
    (
        [u"first line",
         u"  second line",
         u"    third line",
         u"  fourth line"],
        [("p",
          [u"first line",
           u"  second line",
           u"    third line",
           u"  fourth line"],
          u""),
         3]
    ),

    # paragraph with indent
    (
        [u"  first line",
         u"  second line",
         u""],
        [("p",
          [u"  first line",
           u"  second line"],
          u""),
         1]
    ),
    (
        [u"  first line",
         u"  second line"],
        [("p",
          [u"  first line",
           u"  second line"],
          u""),
         1]
    ),
    (
        [u"  first line"],
        [("p",
          [u"  first line"],
          u""),
         0]
    ),
    (
        [u"  first line",
         u"    second line"],
        [("p",
          [u"  first line",
           u"    second line"],
          u""),
         1]
    ),
    (
        [u"  first line",
         u"    second line",
         u"      third line"],
        [("p",
          [u"  first line",
           u"    second line",
           u"      third line"],
          u""),
         2]
    ),
    (
        [u"  first line",
         u"    second line",
         u"      third line",
         u"    fourth line"],
        [("p",
          [u"  first line",
           u"    second line",
           u"      third line",
           u"    fourth line"],
          u""),
         3]
    ),
]



# -*- coding: utf-8 -*-

from nose.tools import *

# functions for test
from ikazuchi.core.translator import BaseTranslator
from ikazuchi.core.translator.utils import *

class TestTranslator(object):

    def setup(self):
        class DummyTranslator(BaseTranslator):
            def __init__(self, lang_from, lang_to, handler):
                pass

            def detect(self, text):
                pass

            def translate(self, text):
                pass

        self.t = DummyTranslator("en", "ja", None)

    def test_parse_html_with_one_span(self):
        html = '<html>この<span class="notranslate">string</span>'\
               'は翻訳されません。</html>'
        actual = self.t.parse_html(unicode(html, "utf-8")).encode("utf-8")
        expected = "この string は翻訳されません。"
        assert_equal(expected, actual)

    def test_parse_html_with_two_span(self):
        html = '<html>この<span class="notranslate">string</span>'\
               'は<span class="notranslate">translate</span></html>'
        expected = "この string は translate"
        actual = self.t.parse_html(unicode(html, "utf-8")).encode("utf-8")
        assert_equal(expected, actual)

    def test_parse_html_with_various(self):
        html = '<html>この<span class="notranslate">string</span>'\
               'は<p>テスト</p>のためで<span>regexp</span>の'\
               'マッチングが<span class="notranslate">correct</span>か'\
               'どうかを<strong>調べます</strong>。</html>'
        expected = "この string は テストのためでregexpの"\
                   "マッチングが correct かどうかを調べます。"
        actual = self.t.parse_html(unicode(html, "utf-8")).encode("utf-8")
        assert_equal(expected, actual)

def test_call_api_with_multithread():
    def dummy_func(num):
        sleep(0.1)
        return num
    from time import sleep
    args = range(1, 30)
    assert_equal(args, call_api_with_multithread(dummy_func, args))



# -*- coding: utf-8 -*-

from nose.tools import *

# functions for test
from ikazuchi.plugins.rstfile import *

class TestreSTParser(object):

    def _test_func(self, data_set, func):
        # common assert function for each test
        _class = self.__class__.__name__
        _func = func.func_name
        _fmt = u"{0}.{1}_{2} ({3} ...)"
        for num, (data, expected) in enumerate(data_set):
            name = _fmt.format(_class, _func, num, str(data)[:10])
            _assert = lambda e, a: assert_equal(e, a)
            _assert.description = name
            yield _assert, expected, list(func(data))

    def test_get_directive(self):
        from data.rst.parse_directive import DATA_SET
        for r in self._test_func(DATA_SET, reSTParser.get_directive):
            yield r

    def test_get_sourceblock(self):
        from data.rst.parse_sourceblock import DATA_SET
        for r in self._test_func(DATA_SET, reSTParser.get_sourceblock):
            yield r

    def test_get_lineblock(self):
        from data.rst.parse_lineblock import DATA_SET
        for r in self._test_func(DATA_SET, reSTParser.get_lineblock):
            yield r

    def test_get_listblock(self):
        from data.rst.parse_listblock import DATA_SET
        for r in self._test_func(DATA_SET, reSTParser.get_listblock):
            yield r

    def test_get_tableblock(self):
        from data.rst.parse_tableblock import DATA_SET
        for r in self._test_func(DATA_SET, reSTParser.get_tableblock):
            yield r

    def test_get_section(self):
        from data.rst.parse_section import DATA_SET
        for r in self._test_func(DATA_SET, reSTParser.get_section):
            yield r

    def test_get_paragraph(self):
        from data.rst.parse_paragraph import DATA_SET
        for r in self._test_func(DATA_SET, reSTParser.get_paragraph):
            yield r


class TestreSTApiCaller(object):

    def __init__(self):
        self.caller = reSTApiCaller(None, "en")
        self._class = self.__class__.__name__
        self._fmt = u"{0}.{1}_{2}"

    def _test_func(self, data_set, func):
        # common assert function for each test
        _func = func.func_name
        for num, (data, expected) in enumerate(data_set):
            name = self._fmt.format(self._class, _func, num)
            _assert = lambda e, a: assert_equal(e, a)
            _assert.description = name
            yield _assert, expected, func(data)

    def _call_func(self, data_set, func):
        # common assert function for each test calling api method
        def dummy(text):
            # return given text as is
            return None, text

        _func = func.func_name
        for num, data in enumerate(data_set):
            args, expected = data[:-1], data[-1]
            name = self._fmt.format(self._class, _func, num, str(data)[:10])
            _, actual = func(dummy, *args)
            _assert = lambda e, a: assert_equal(e, a)
            _assert.description = name
            yield _assert, expected, actual

    def test_get_table_column_width(self):
        from data.rst.api_call_get_table_column_width import DATA_SET
        _func = u"test_get_table_column_width"
        for num, (items, width, expected) in enumerate(DATA_SET):
            actual = reSTApiCaller.get_table_column_width(items, width)
            name = self._fmt.format(self._class, _func, num, str(items)[:10])
            _assert = lambda e, a: assert_equal(e, a)
            _assert.description = name
            yield _assert, expected, actual

    def test_get_indent_and_text(self):
        from data.rst.api_call_text_with_indent import DATA_SET
        for r in self._test_func(DATA_SET, reSTApiCaller.get_indent_and_text):
            yield r

    def test_markup_paragraph_notranslate(self):
        from data.rst.api_call_markup_notranslate import DATA_SET
        for r in self._test_func(DATA_SET,
                    reSTApiCaller.markup_paragraph_notranslate):
            yield r

    def test_split_text_into_multiline(self):
        self.caller.lang_to = "en"
        from data.rst.api_call_split_text import DATA_SET
        _func = self.caller.split_text_into_multiline
        for r in self._test_func(DATA_SET, _func):
            yield r

    def test_split_text_into_multiline_ja(self):
        self.caller.lang_to = "ja"
        from data.rst.api_call_split_text_ja import DATA_SET
        _func = self.caller.split_text_into_multiline
        for r in self._test_func(DATA_SET, _func):
            yield r

    def test_call_for_directive(self):
        from data.rst.api_call_directive import DATA_SET
        for r in self._call_func(DATA_SET, self.caller._call_for_directive):
            yield r

    def test_call_for_sourceblock(self):
        from data.rst.api_call_sourceblock import DATA_SET
        for r in self._call_func(DATA_SET, self.caller._call_for_sourceblock):
            yield r

    def test_call_for_lineblock(self):
        from data.rst.api_call_lineblock import DATA_SET
        for r in self._call_func(DATA_SET, self.caller._call_for_lineblock):
            yield r

    def test_call_for_listblock(self):
        from data.rst.api_call_listblock import DATA_SET
        for r in self._call_func(DATA_SET, self.caller._call_for_listblock):
            yield r

    def test_call_for_tableblock(self):
        from data.rst.api_call_tableblock import DATA_SET
        for r in self._call_func(DATA_SET, self.caller._call_for_tableblock):
            yield r

    def test_call_for_section(self):
        from data.rst.api_call_section import DATA_SET
        for r in self._call_func(DATA_SET, self.caller._call_for_section):
            yield r

    def test_call_for_paragraph(self):
        from data.rst.api_call_paragraph import DATA_SET
        for r in self._call_func(DATA_SET, self.caller._call_for_paragraph):
            yield r



# -*- coding: utf-8 -*-

DATA_SET = [
    # normal paragraph
    (  # 0
        [u"*first line*"],

        [u"<span class=notranslate>*first line*</span>\n"]
    ),
    (  # 1
        [u"first line",
         u"**second line**"],

        [u"first line <span class=notranslate>**second line**</span>\n"]
    ),
    (  # 2
        [u"first line",
         u"second line",
         u"``third line``"],

        [u"first line second line "\
         u"<span class=notranslate>``third line``</span>\n"]
    ),
    (  # 3
        [u"first line",
         u"  **second line**"],

        [u"first line\n",
         u"  <span class=notranslate>**second line**</span>\n"]
    ),
    (  # 4
        [u"first line",
         u"  second line",
         u"    ``third line``"],

        [u"first line\n",
         u"  second line\n",
         u"    <span class=notranslate>``third line``</span>\n"]
    ),
    (  # 5
        [u"first line",
         u"  second line",
         u"    third line",
         u"  fourth line"],

        [u"first line\n",
         u"  second line\n",
         u"    third line\n",
         u"  fourth line\n"],
    ),
    (   # 6
        # auto line break with textwrap
        [u"long long long long long long long long long long first line",
         u"looooooooooooooooooooooooooooooooooooooooong **second line**"],

        [u"long long long long long long long long long long first line\n"\
          "looooooooooooooooooooooooooooooooooooooooong <span\n"\
          "class=notranslate>**second line**</span>\n"]
    ),
    (   # 7
        # role
        [u":mod:`ikazuchi` has some functions, ",
         u":func:`translate` and :func:`detect`."],

        [u"<span class=notranslate>:mod:`ikazuchi`</span> has some "\
         u"functions,\n<span class=notranslate>:func:`translate`</span> "\
         u"and <span\nclass=notranslate>:func:`detect`</span>.\n"]
    ),

    # paragraph with indent
    (  # 8
        [u"  *first line*"],

        [u"  <span class=notranslate>*first line*</span>\n"]
    ),
    (  # 9
        [u"  first line",
         u"  **second line**"],

        [u"  first line <span class=notranslate>**second line**</span>\n"]
    ),
    (  # 10
        [u"  first line",
         u"  second line",
         u"  ``third line``"],

        [u"  first line second line "\
         u"<span class=notranslate>``third line``</span>\n"]
    ),
    (  # 11
        [u"  first line",
         u"    **second line**"],

        [u"  first line\n",
         u"    <span class=notranslate>**second line**</span>\n"]
    ),
    (  # 12
        [u"  first line",
         u"    second line",
         u"      ``third line``"],

        [u"  first line\n",
         u"    second line\n",
         u"      <span class=notranslate>``third line``</span>\n"]
    ),
    (  # 13
        [u"  first line",
         u"    second line",
         u"      third line",
         u"    fourth line"],

        [u"  first line\n",
         u"    second line\n",
         u"      third line\n",
         u"    fourth line\n"],
    ),
    (   # 14
        # auto line break with textwrap
        [u"  long long long long long long long long long long first line",
         u"  looooooooooooooooooooooooooooooooooooooooong **second line**"],

        [u"  long long long long long long long long long long first line\n"\
          "  looooooooooooooooooooooooooooooooooooooooong <span\n"\
          "  class=notranslate>**second line**</span>\n"]
    ),
    (   # 15
        # role
        [u"  :mod:`ikazuchi` has some functions, ",
         u"  :func:`translate` and :func:`detect`."],

        [u"  <span class=notranslate>:mod:`ikazuchi`</span> has some "\
         u"functions,\n  <span class=notranslate>:func:`translate`</span> "\
         u"and <span\n  class=notranslate>:func:`detect`</span>.\n"]
    ),
]



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        [u"| first line",
         u"| second line",
         u""],
        [("ln",
          [u"| first line",
           u"| second line"],
          u""),
         1]
    ),
    (
        [u"| first line",
         u"| second line"],
        [("ln",
          [u"| first line",
           u"| second line"],
          u""),
         1]
    ),
    (
        [u"| first line",
         u"|   second line",
         u""],
        [("ln",
          [u"| first line",
           u"|   second line"],
          u""),
         1]
    ),
    (
        [u"| first line",
         u"|   second line",
         u"|     third line",
         u""],
        [("ln",
          [u"| first line",
           u"|   second line",
           u"|     third line"],
          u""),
         2]
    ),
    (
        [u"| first line",
         u"|   second line",
         u"|     third line",
         u"|   fourth line",
         u""],
        [("ln",
          [u"| first line",
           u"|   second line",
           u"|     third line",
           u"|   fourth line"],
          u""),
         3]
    ),
    (
        [u"| first line",
         u"|",
         u"|   second line",
         u""],
        [("ln",
          [u"| first line",
           u"|",
           u"|   second line"],
          u""),
         2]
    ),
    (
        [u"| first line",
         u"|",
         u"|   second line",
         u"|",
         u""],
        [("ln",
          [u"| first line",
           u"|",
           u"|   second line",
           u"|"],
          u""),
         3]
    ),
]



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        [u"* first line",
         u""],
        [("ls",
          [u"* first line",
           u""],
          u""),
         1]
    ),
    (
        [u"* first line"],
        [("ls",
          [u"* first line"],
          u""),
         1]
    ),
    (
        [u"* first line",
         u"* second line",
         u""],
        [("ls",
          [u"* first line",
           u"* second line",
           u""],
          u""),
         2]
    ),
    (
        [u"* first line",
         u"* second line"],
        [("ls",
          [u"* first line",
           u"* second line"],
          u""),
         1]
    ),
    (
        [u"* first line",
         u"  * second line",
         u""],
        [("ls",
          [u"* first line",
           u"  * second line",
           u""],
          u""),
         2]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  * second line",
           u""],
          u""),
         3]
    ),
    (
        [u"* first line",
         u"  * second line",
         u"    * third line",
         u""],
        [("ls",
          [u"* first line",
           u"  * second line",
           u"    * third line",
           u""],
          u""),
         3]
    ),
    (
        [u"* first line",
         u"  * second line",
         u"    sub line",
         u"    * third line",
         u""],
        [("ls",
          [u"* first line",
           u"  * second line",
           u"    sub line",
           u"    * third line",
           u""],
          u""),
         4]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"",
         u"    * third line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  * second line",
           u"",
           u"    * third line",
           u""],
          u""),
         5]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"    sub line",
         u"",
         u"    * third line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  * second line",
           u"    sub line",
           u"",
           u"    * third line",
           u""],
          u""),
         6]
    ),
    (
        [u"* first line",
         u"",
         u"  * second line",
         u"",
         u"",
         u"    * third line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  * second line",
           u""],
          u""),
         3]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"  - third line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  - second line",
           u"  - third line",
           u""],
          u""),
         4]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    - third line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  - second line",
           u"",
           u"    - third line",
           u""],
          u""),
         5]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    * third line",
         u"",
         u"  - fourth line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  - second line",
           u"",
           u"    * third line",
           u"",
           u"  - fourth line",
           u""],
          u""),
         7]
    ),
    (
        [u"* first line",
         u"",
         u"  - second line",
         u"",
         u"    * third line",
         u"    * fourth line",
         u"",
         u"  - fifth line",
         u""],
        [("ls",
          [u"* first line",
           u"",
           u"  - second line",
           u"",
           u"    * third line",
           u"    * fourth line",
           u"",
           u"  - fifth line",
           u""],
          u""),
         8]
    ),
]



# -*- coding: utf-8 -*-

from minimock import mock, Mock, restore
from nose.tools import *

# functions for test
from ikazuchi.vim import (raimei, utils)

class TestRaimei(object):

    eval_var = {
        "raimei_api": "google",
        "raimei_from": "en",
        "raimei_to": "ja",
        "&enc": "utf-8",
    }

    def setup(self):
        raimei.vim = None
        mock("raimei.vim")
        raimei.vim.current = Mock("current",
            window=Mock(
                "window",
                cursor=Mock("cursor"),
            ),
            range=Mock(
                "range",
                start=Mock("start"),
                end=Mock("end"),
            ),
        )
        utils.vim = None
        mock("utils.vim")
        _eval_func = lambda var: self.eval_var[var]
        utils.vim.eval = Mock("eval", returns_func=_eval_func)

    def teardown(self):
        restore()

    def test_get_vim_variables(self):
        ret = (
            self.eval_var["raimei_api"],
            self.eval_var["raimei_from"],
            self.eval_var["raimei_to"],
            self.eval_var["&enc"],
        )
        assert_equal(ret, raimei.get_vim_variables())

    def test_get_target_range(self):
        class Buffer(Mock):
            buf = ["first line", "second line", "third line",
                   "fourth line", "fifth line", ""]

            def __len__(self):
                return len(self.buf)

            def __getitem__(self, i):
                return self.buf[i]

        raimei.vim.current.buffer = Buffer("buffer")
        raimei.vim.current.range.start = 2
        raimei.vim.current.range.end = 2
        raimei.vim.current.window.cursor = (3, 0)
        assert_equal((2, 5), raimei.get_target_range())
        # other line on the cursor, only single line range is specified
        raimei.vim.current.window.cursor = (1, 0)
        assert_equal((2, 3), raimei.get_target_range())

    def test_get_target_range_infinite(self):
        class Buffer(Mock):
            buf = ["first line", "second line", "third line"]

            def __len__(self):
                return len(self.buf)

            def __getitem__(self, i):
                return self.buf[i]

        raimei.vim.current.buffer = Buffer("buffer")
        # from 1st line
        raimei.vim.current.range.start = 0
        raimei.vim.current.range.end = 0
        raimei.vim.current.window.cursor = (1, 0)
        assert_equal((0, 3), raimei.get_target_range())
        # from 2nd line
        raimei.vim.current.range.start = 1
        raimei.vim.current.range.end = 1
        raimei.vim.current.window.cursor = (2, 0)
        assert_equal((1, 3), raimei.get_target_range())



# -*- coding: utf-8 -*-

from base import BaseHandler, LanguageHandler, NullHandler
from text import SingleSentenceHandler

__all__ = [
    "LanguageHandler",
    "NullHandler",
    "SingleSentenceHandler",
]



# -*- coding: utf-8 -*-

import sys
from nose.tools import *
from StringIO import StringIO

# functions for test
import ikazuchi.utils


def test_encoding_action():
    class Namespace(object):
        encoding = None
    namespace = Namespace()
    a = ikazuchi.utils.EncodingAction(['-e', '--encoding'], dest='encoding')
    a(None, namespace, "utf-8", None)
    assert_equal(["utf-8", "utf-8"], namespace.encoding)
    a(None, namespace, "utf-8, euc-jp", None)
    assert_equal(["utf-8", "euc-jp"], namespace.encoding)

def test_check_encoding():
    errs = ikazuchi.utils.check_encoding(["euc-jp", "utf-8"])
    assert_equals([], errs)
    errs = ikazuchi.utils.check_encoding(["unknown", "utf-8"])
    assert_equals(["unknown"], errs)
    errs = ikazuchi.utils.check_encoding(["utf-8", "unknown"])
    assert_equals(["unknown"], errs)
    errs = ikazuchi.utils.check_encoding(["unknown", "notexist"])
    assert_equals(["unknown", "notexist"], errs)

def test_get_command():
    import platform
    os_name = platform.system()
    if os_name == "Windows":
        # FIXME: tell me appropriate command
        pass
    else:
        assert_equal(["/bin/ls"], list(ikazuchi.utils.get_command("ls")))

def test_python_version():
    data = [(2, 6, 4, 'final', 0), (2, 5, 1, 'final', 0)]
    sys.stdout = StringIO()
    for ver in data:
        sys.version_info = ver
        try:
            ikazuchi.utils.check_python_version()
            assert_equals("", sys.stdout.getvalue())
        except SystemExit:
            assert_equals(ikazuchi.utils._UNSUPPORTED_VERSION,
                            sys.stdout.getvalue().rstrip('\r\n'))



# -*- coding: utf-8 -*-

from minimock import mock, Mock, restore
from nose.tools import *

# functions for test
from ikazuchi.vim import enrai

class TestEnrai(object):

    eval_var = {
        "raimei_api": "google",
        "raimei_from": "en",
        "raimei_to": "ja",
        "&enc": "utf-8",
    }

    test_text = " \t   Enrai is translated   as 'Distant thunder'."

    def setup(self):
        enrai.vim = None
        mock("enrai.vim")
        _eval_func = lambda var: self.eval_var[var]
        enrai.vim.eval = Mock("eval", returns_func=_eval_func)
        enrai.vim.current = Mock("current",
            window=Mock("window"),
            line=Mock(
                "line",
                split=Mock("split", returns=self.test_text.split(" ")),
            ),
        )

    def teardown(self):
        restore()

    def test_get_word_on_cursor_1(self):
        _enc = self.eval_var["&enc"]
        enrai.vim.current.window.cursor = (1, 6)
        assert_equal("Enrai", enrai.get_word_on_cursor(_enc))

    def test_get_word_on_cursor_2(self):
        _enc = self.eval_var["&enc"]
        enrai.vim.current.window.cursor = (1, 12)
        assert_equal("is", enrai.get_word_on_cursor(_enc))

    def test_get_word_on_cursor_3(self):
        _enc = self.eval_var["&enc"]
        enrai.vim.current.window.cursor = (1, 32)
        assert_equal("'Distant", enrai.get_word_on_cursor(_enc))



# -*- coding: utf-8 -*-

try:
    import vim
except ImportError:
    pass

# OVERWRITE THIS VARIABLES WITH YOUR APIKEY
GOOGLE_APIKEY = "AIzaSyDDCHHwbfHLIsHWEhxAu41UmrRCg_Xmvm8"
MICROSOFT_APIKEY = "D9D0E326A70EA4E66218F43130890052808A0142"

def to_unicode(seq, enc):
    return [unicode(i, enc) for i in seq]

def to_encode(seq, enc):
    return [i.encode(enc) for i in seq]

def get_vim_variables():
    # settings from VimScript(.vimrc)
    try:
        api_name = vim.eval("raimei_api")
        lang_from = vim.eval("raimei_from")
        lang_to = vim.eval("raimei_to")
        enc = vim.eval("&enc")
    except vim.error:
        #  vim.error is String Exception
        print "Set enc, raimei_api, raimei_from, raimei_to variables!"
        raise
    return api_name, lang_from, lang_to, enc

def get_apikey(api_name):
    apikey = ""
    if api_name == "google":
        apikey = GOOGLE_APIKEY
    elif api_name == "microsoft":
        apikey = MICROSOFT_APIKEY
    return apikey

def get_translate_method(translater, api_name):
    api_method = lambda api, texts: (api, texts)
    if api_name == "google":
        api_method = translater.translate
    elif api_name == "microsoft":
        api_method = translater.translate_array
    return api_method



# -*- coding: utf-8 -*-

class IkazuchiError(Exception):
    """ ikazuchi root exception """
    pass

class TranslatorError(IkazuchiError):
    """ ikazuchi translator exception """
    pass

class NeedApiKeyError(TranslatorError): pass



# -*- coding: utf-8 -*-

from nose.tools import *

# functions for test
from ikazuchi.core.translator import TranslatingGoogle

class TestGoogleTranslator(object):
    """
    Do not test to request actually since Google Translate API v2 is
    a paid service from 2011/12/1
    """

    def setup(self):
        self.t = TranslatingGoogle("ja", "en", None)

    def test_get_url(self):
        def detect():
            pass

        expected = "https://www.googleapis.com/language/translate/v2/detect"
        assert_equal(expected, self.t.get_url(detect))

    @nottest
    def test_detect(self):
        res = self.t.detect([unicode("テスト", "utf-8")])
        assert_equal(u"ja", res[1][0][0][u"language"])

    @nottest
    def test_translate(self):
        assert_equal(('Google', [u'Test']),
                     self.t.translate([unicode("テスト", "utf-8")]))



# -*- coding: utf-8 -*-

DATA_SET = [
    (   # 0 no indent line
         u"no indent line.",

        [u"no indent line.\n"],
    ),
    (   # 1 indent line
         u"  indent line.",

        [u"  indent line.\n"],
    ),
    (   # 2 no indent long line
         u"no indent long long long long long long long long long long "\
         u"long long long long long long long long long long long long "\
         u"long long long long line.",

        [u"no indent long long long long long long long long long long "\
         u"long long\nlong long long long long long long long long long "\
         u"long long long long\nline.\n"]
    ),
    (   # 3 indent long line
         u"  indent long long long long long long long long long long "\
         u"long long long long long long long long long long long long "\
         u"long long long long line.",

        [u"  indent long long long long long long long long long long "\
         u"long long\n  long long long long long long long long long long "\
         u"long long long\n  long line.\n"]
    ),
]



# -*- coding: utf-8 -*-

import codecs
import re
import string
import sys
import textwrap

from ikazuchi.core.handler.base import BaseHandler
from ikazuchi.core.handler.utils import *
from ikazuchi.ikazuchi import (base_parser, subparsers)

# for Debug
import logging
log = logging.getLogger(__name__)

try:
    from ikazuchi.locale import _
except ImportError:
    def _(s):
        return s

_INLINE = [
    "\*+.*?\*+",    # italic or bold
    "`+.*?`+",      # literal
]

_ROLE = [
    ":.*?:`.*?`",   # :role:`xxx`
]

_HYPER_LINK = [
    "https?://[^ ]+",               # http://xxx
    "`.*?`__?",                     # `link <http://>`_ , `link`_ or `name`__
    "^\.\.\s*_.*?:\s*https?://[^ ]+$",  # .. _link: http://xxx
    "^__\s*https?://[^ ]+$",        # __ http://xxx
]

_RUBRIC = [
    "\[#.*\]_",                 # [#f1]_
    "^\.\.\s*\[#.*\].*$",       # .. [#f1] description
]

_REFERENCE = [
    "\[[^#].*?\]_",             # [ref]_
    "^\.\.\s*\[[^#]*?\].*$",    # .. [ref] description
]

_SOURCE = [
    ":\s*$",    # collon:
    ":\s+",     # collon: [space]
    "::\s*",    # source::
]

# FIXME: it need to match first _HYPERLINK than _LILINE
#      : because, `link`_ is matched in preference to `literal`
_NOTRANSLATE = _ROLE + _HYPER_LINK + _INLINE + _RUBRIC + _REFERENCE + _SOURCE
_NOTRANSLATE_PTRN = re.compile(r"({0})".format("|".join(_NOTRANSLATE)),
                               re.M | re.U)

_SECTION = re.compile(r"""
    (?P<over_line>[%(symbol)s]{2,}\s+)?    # =======
    (?P<section>[^%(symbol)s]+?\s+)        # section
    (?P<under_line>[%(symbol)s]{2,}\s+)$   # =======
""" % {"symbol": string.punctuation}, re.U | re.X)

_SECTION_LINE = re.compile(r'^[%s]{2,}\s+' % string.punctuation, re.U)

_LISTBLOCK = re.compile(r"""(
      ^[*\-\d#]\.*\s+       # * list
    | ^\s+[*\-\d#]\.*\s+    #   * nested list
)(.*?)$""", re.U | re.X)

_LINEBLOCK = re.compile(r"""
      (^\|\s*)(.*?)$    # "| line block" or "|"
""", re.U | re.X)

_TABLEBLOCK = re.compile(r"""(
      (?P<grid_rule>^\s*\+([\-=]+\+)+\s*)       # grid table rule
    | (?P<grid_rows>^\s*\|(.*?\|)+\s*)          # grid table rows
    | (?P<simple_rule>^\s*=+(\s+=+){1,}\s*)     # simple table rule
    | (?P<simple_rows>^\s*.*?(\s+.*?){1,}\s*)   # simple table rows
)$""", re.U | re.X)

_SOURCE_CODE = re.compile(r"^.*?::\s*$", re.U)

_DIRECTIVE = re.compile(r"""(
      {0}               # source code
    | ^\.\..*(?<!::)    # comment
    | ^\.\.\s+.+::.*    # .. xxx:: or .. xxx:: yyy
)$""".format(_SOURCE_CODE), re.U | re.X)

_DIRECTIVE_WITH_PARAGRAPH = re.compile(r"""(
      ^\.\.\s+note::        # .. note::
    | ^\.\.\s+warning::     # .. warning::
    | ^\.\.\s+seealso::     # .. seealso::
    | ^\.\.\s+rubric::      # .. rubric::
    | ^\.\.\s+tip::         # .. tip::
    | ^\.\.\s+error::       # .. error::
    | ^\.\.\s+hint::        # .. hint::
    | ^\.\.\s+important::   # .. important::
    | ^\.\.\s+attention::   # .. attention::
    | ^\.\.\s+caution::     # .. caution::
    | ^\.\.\s+danger::      # .. danger::
).*$""", re.U | re.X)

_EMPTY_LINE = re.compile(r"^\s*$", re.U)
_LINE_WITH_INDENT = re.compile(r"(^\s+)(.+?)$", re.U)
_LINE_WITH_INDENT_OR_NONE = re.compile(r"(^\s*)(.+?)$", re.U)
_PARAGRAPH_START = re.compile(r"^([\S\.]+)(.*?)$", re.U)
_END_OF_SENTENCE = {
    "en": re.compile(unicode(r".*?[\.\?!]", "utf-8"), re.M | re.U),
    "ja": re.compile(unicode(r".*?[。．？！]", "utf-8"), re.M | re.U),
}

REST_BLOCK_TYPE = {
    "directive": "d",
    "lineblock": "ln",
    "listblock": "ls",
    "paragraph": "p",
    "tableblock": "t",
    "section": "se",
    "source": "so",
}


# argument parser for rstfile
rst_parser = subparsers.add_parser("rstfile", parents=[base_parser])
rst_parser.set_defaults(rst_file=None, output="output.rst")
rst_parser.add_argument(dest="rst_file", help="target rst file")
rst_parser.add_argument("-o", "--output", dest="output", metavar="OUTPUT",
    help="translated output file name, default is 'output.rst'")

class Handler(BaseHandler):
    """
    Handler class for translating reST file
    """
    def __init__(self, opts):
        self.api = opts.api if opts.api else "microsoft"
        if self.api == "microsoft":
            self.method_name = "translate_array"
        self.rst_file = get_and_check_file_access(opts.rst_file)
        self.output = get_and_check_file_access(opts.output, "w")
        self.encoding = opts.encoding
        with codecs.open(self.rst_file, mode="r",
                         encoding=self.encoding[1]) as f:
            lines = f.readlines()  # read out all lines
        blocks = reSTParser(lines).parse()
        self.caller = reSTApiCaller(blocks, opts.lang_to)

    def _show_progress(self, lines):
        sys.stdout.write("." * len(lines))
        sys.stdout.flush()

    def _call_method(self, api_method):
        in_enc, out_enc = self.encoding
        with codecs.open(self.output, mode="w", encoding=out_enc) as f:
            for lines in self.caller.call(api_method):
                f.writelines(lines)
                self._show_progress(lines)
        sys.stdout.write("\n")
        sys.stdout.write("Translated by {0}\n".format(self.api.title()))


class reSTParser(object):
    """
    Parser class for reading/parsing lines with reST format
    """
    def __init__(self, lines):
        self.lines = lines

    def _convert_lines_to_blocks(self, lines):
        blocks = []
        skip_num = 0
        for num, line in enumerate(lines):
            if skip_num > 0:
                skip_num -= 1
                continue
            _lines = lines[num:]
            info, skip_num = self.get_directive(_lines)
            if not info[0]:
                info, skip_num = self.get_sourceblock(_lines)
            if not info[0]:
                info, skip_num = self.get_lineblock(_lines)
            if not info[0]:
                info, skip_num = self.get_listblock(_lines)
            if not info[0]:
                info, skip_num = self.get_tableblock(_lines)
            if not info[0]:
                info, skip_num = self.get_section(_lines)
            if not info[0]:
                info, skip_num = self.get_paragraph(_lines)
            if not info[0]:  # others
                info = (None, [line], None)
            blocks.append(info)
        return blocks

    def parse(self):
        return self._convert_lines_to_blocks(self.lines)

    @classmethod
    def get_directive(self, lines):
        btype, block, directive, bnum = None, [], u"", 0
        match = re.search(_DIRECTIVE, lines[0])
        if match:
            btype = REST_BLOCK_TYPE["directive"]
            directive = match.groups()[0]
            bnum, block = 0, lines[0:1]
            if not lines[1:2][0:2] == "..":
                _cmp = lambda line: re.search(_PARAGRAPH_START, line)
                bnum, block = get_sequential_block(lines, _cmp)
        return (btype, block, directive), bnum

    @classmethod
    def get_sourceblock(self, lines):
        def _get_code_block(_lines):
            btype, first, num = None, u"", 0
            if len(_lines) >= 2:
                for num, mline in enumerate(get_multiline(_lines, 2)):
                    if not btype and re.search(_SOURCE_CODE, mline[0]) and (
                        re.search(_EMPTY_LINE, mline[1]) or
                        re.search(_LINE_WITH_INDENT, mline[1])
                    ):
                        btype = REST_BLOCK_TYPE["source"]
                        first = u"".join(_lines[0:num + 1])
                    elif re.search(_EMPTY_LINE, mline[0]) and \
                         not re.search(_LINE_WITH_INDENT, mline[1]):
                        break
                else:
                    # maybe read out to EOF
                    num += 1
            return (btype, _lines[0:num + 1], first), num

        return _get_code_block(lines)

    @classmethod
    def get_lineblock(self, lines):
        btype, block, bnum = None, [], 0
        if re.search(_LINEBLOCK, lines[0]):
            btype = REST_BLOCK_TYPE["lineblock"]
            _cmp = lambda line: not re.search(_LINEBLOCK, line)
            bnum, block = get_sequential_block(lines, _cmp)
        return (btype, block, u""), bnum

    @classmethod
    def get_listblock(self, lines):
        def _get_code_block(_lines):
            num = 0
            for num, mline in enumerate(get_multiline(_lines, 2)):
                if re.search(_LISTBLOCK, mline[0]) or \
                   re.search(_LINE_WITH_INDENT, mline[0]) or (
                    re.search(_EMPTY_LINE, mline[0]) and
                    re.search(_LISTBLOCK, mline[1])
                ):
                    pass
                else:
                    break
            else:
                # maybe read out to EOF
                num += 1
            return num, _lines[0:num + 1]

        btype, block, bnum = None, [], 0
        match = re.search(_LISTBLOCK, lines[0])
        if match:
            btype = REST_BLOCK_TYPE["listblock"]
            bnum, block = _get_code_block(lines)
        return (btype, block, u""), bnum

    @classmethod
    def get_tableblock(self, lines):
        btype, block, bnum = None, [], 0
        match = re.search(_TABLEBLOCK, lines[0])
        if match and not match.groupdict().get("simple_rows"):
            btype = REST_BLOCK_TYPE["tableblock"]
            _cmp = lambda line: re.search(_EMPTY_LINE, line)
            bnum, block = get_sequential_block(lines, _cmp)
        return (btype, block, u""), bnum

    @classmethod
    def get_section(self, lines):
        def _get_code_block(_lines):
            num, section = 0, []
            for mline in get_multiline(_lines, 3):
                match = re.search(_SECTION, "".join(mline))
                if re.search(_EMPTY_LINE, mline[0]) or (
                   mline[1:] and re.search(_EMPTY_LINE, mline[1])):
                    break
                elif match:
                    d = match.groupdict()
                    if d.get("section"):
                        if d.get("over_line") or d.get("under_line"):
                            num, section = 2, mline
                            break
            return num, section

        btype = None
        bnum, block = _get_code_block(lines)
        if bnum > 0:
            btype = REST_BLOCK_TYPE["section"]
        return (btype, block, u""), bnum

    @classmethod
    def get_paragraph(self, lines):
        btype, block, bnum = None, [], 0
        if re.search(_PARAGRAPH_START, lines[0]) or \
           re.search(_LINE_WITH_INDENT, lines[0]):
            btype = REST_BLOCK_TYPE["paragraph"]
            _cmp = lambda line: re.search(_EMPTY_LINE, line)
            bnum, block = get_sequential_block(lines, _cmp)
        return (btype, block, u""), bnum


class reSTApiCaller(object):
    """
    Caller class for converting reST block calling api method
    """
    max_query = 100

    def __init__(self, blocks, lang_to):
        self.blocks = blocks
        self.lang_to = lang_to

    def split_text_into_multiline(self, text):
        # FIXME: only valid for paragraph, need for other blocks
        if self.lang_to in ("ja"):
            def set_indent(lines):
                indent, _text = self.get_indent_and_text(lines[0])
                lines[0] = _text  # replace first line with text removed indent
                _lines = []
                for line in lines:
                    line = line[1:] if line[:1] == " " else line
                    _lines.append(u"{0}{1}\n".format(indent, line))
                return _lines

            text = text.rstrip()
            eos_ptrn = _END_OF_SENTENCE.get(self.lang_to)
            lines = re.findall(eos_ptrn, text)
            if not lines:
                lines = [text]
            else:
                extra_text = text[len(u"".join(lines)):]
                if extra_text:
                    lines.append(extra_text)
            return set_indent(lines)
        else:
            indent, _ = self.get_indent_and_text(text)
            dedented_text = textwrap.dedent(text).strip()
            indented = textwrap.fill(dedented_text, initial_indent=indent,
                                        subsequent_indent=indent)
            return [u"{0}\n".format(indented)]

    def split_lines_with_eos(self, lines):
        _lines = []
        for line in lines:
            _lines.extend(self.split_text_into_multiline(line))
        return _lines

    def _markup_notranslate(self, line, match):
        prefix, text = "", line
        if match:
            prefix, text = match.groups()
        return prefix, self.markup_paragraph_notranslate(text)

    def _call_keeping_prefix(self, api_method, line, match):
        prefix, text = "", line
        if match:
            prefix, text = match.groups()
        _text = self.markup_paragraph_notranslate(text)
        api, result = api_method([_text])
        return api, u"{0}{1}\n".format(prefix, result[0])

    def _call_keeping_prefix_with_array(self, api_method, indents, lines):
        api, trans = None, []
        for pos in range(0, len(indents), self.max_query):
            _idns = indents[pos:pos + self.max_query]
            _lines = lines[pos:pos + self.max_query]
            api, _trans = api_method(_lines)
            trans.extend([u"{0}{1}\n".format(*i) for i in zip(_idns, _trans)])
        return api, trans

    def _call_for_directive(self, api_method, block_lines, first):
        if not re.search(_DIRECTIVE_WITH_PARAGRAPH, first):
            return None, block_lines

        # FIXME: need to check list/line block concatenating ...
        api, indents, lines = None, [], []
        for line in concatenate_lines(block_lines[1:], _LINE_WITH_INDENT):
            match = re.search(_LINE_WITH_INDENT, line)
            indent, line = self._markup_notranslate(line, match)
            indents.append(indent)
            lines.append(line)
        api, trans = self._call_keeping_prefix_with_array(
                                api_method, indents, lines)
        return api, block_lines[:1] + trans

    def _call_for_sourceblock(self, api_method, block_lines, first):
        if re.search(r"^::\s*$", first):
            api, lines = None, block_lines
        else:
            _lines = first.split("\n")
            api, lines = self._call_for_paragraph(api_method, _lines[:-1])
            lines.extend(block_lines[len(_lines) - 1:])
        return api, lines

    def _call_for_lineblock(self, api_method, block_lines):
        api, indents, lines = None, [], []
        for line in block_lines:
            match = re.match(_LINEBLOCK, line)
            indent, line = self._markup_notranslate(line, match)
            indents.append(indent)
            lines.append(line)
        return self._call_keeping_prefix_with_array(api_method, indents, lines)

    def _call_for_listblock(self, api_method, block_lines):
        def _concatenate_lines(lines):
            def _extend(lines, line, prev):
                _lines = [line]
                if prev:
                    _lines.insert(0, prev)
                lines.extend(_lines)

            if len(lines) < 2:
                return lines

            # FIXME: help to be more simple!
            _lines, _prev = [], u""
            lines.append("")  # add dummy line to check last line as mline[0]
            for mline in get_multiline(lines, 2):
                if re.search(_LISTBLOCK, mline[0]):
                    if not re.search(_LISTBLOCK, mline[1]) and \
                       re.search(_LINE_WITH_INDENT, mline[1]):
                        if _prev:
                            _lines.append(_prev)
                        _prev = mline[0]
                    else:
                        _extend(_lines, mline[0], _prev)
                        _prev = u""
                elif re.search(_EMPTY_LINE, mline[0]):
                    _extend(_lines, mline[0], _prev)
                    _prev = u""
                else:
                    _prev = u"{0} {1}".format(_prev.rstrip(),
                                              mline[0].lstrip())
            else:
                if _prev:
                    _lines.append(_prev)
            return _lines

        api, indents, lines = None, [], []
        for line in _concatenate_lines(block_lines):
            match = re.match(_LISTBLOCK, line)
            indent, line = self._markup_notranslate(line, match)
            indents.append(indent)
            lines.append(line)
        return self._call_keeping_prefix_with_array(api_method, indents, lines)

    def _call_for_gridtable(self, api_method, block_lines, column_length):
        api, results = None, []
        east_asian_width, max_width = [], []
        for line in block_lines:
            indent, line = self.get_indent_and_text(line)
            d = re.search(_TABLEBLOCK, line).groupdict()
            if d.get("grid_rule"):
                items = line.split("+")[1:-1]
                _east = []  # dummy
            else:  # it means d.get("grid_rows"):
                _items = [i.strip() for i in line.split("|")]
                _items = map(self.markup_paragraph_notranslate, _items[1:-1])
                api, items = api_method(_items)
                _east, max_width = self.get_table_column_width(
                                            items, max_width)
            east_asian_width.append(_east)
            results.append(items)

        # format table block considering its width
        lines = []
        rule_fmt = u"#+#".join(u"{%s:{%s}}" % (num, num + 1)
                            for num in range(0, len(max_width) * 2, 2))
        rule_fmt = u"{0}+#{1}#+\n".format(indent, rule_fmt)
        row_fmt = u" | ".join(u"{%s:{%s}}" % (num, num + 1)
                            for num in range(0, len(max_width) * 2, 2))
        row_fmt = u"{0}| {1} |\n".format(indent, row_fmt)
        for rnum, row in enumerate(results):
            _width = list(max_width)  # copy
            len_width = map(len, row)
            rule_flag = False
            if re.search(r"[\-=]+\s*", row[0], re.U):
                rule_flag = True
            for cnum, col in enumerate(row):
                if rule_flag:
                    row[cnum] = col[0] * max_width[cnum]
                else:
                    _width[cnum] = max_width[cnum] - (
                        east_asian_width[rnum][cnum] - len_width[cnum])
            if rule_flag:
                _line = rule_fmt.format(*zip_with_flatlist(row, _width))
                _line = _line.replace("#", row[0][0])
            else:
                _line = row_fmt.format(*zip_with_flatlist(row, _width))
            lines.append(_line)
        return api, lines

    def _call_for_simpletable(self, api_method, block_lines, column_length):
        api, results = None, []
        east_asian_width, max_width = [], []
        for line in block_lines:
            indent, line = self.get_indent_and_text(line)
            d = re.search(_TABLEBLOCK, line).groupdict()
            if d.get("simple_rule"):
                items = line.split()
                simple_rule_width = map(len, items)
                _east = []  # dummy
            else:  # it measn d.get("simple_rows"):
                _items = []
                for column_width in simple_rule_width:
                    _text = line[0:column_width].strip()
                    _items.append(self.markup_paragraph_notranslate(_text))
                    line = line[column_width:].strip()  # overwrite line
                api, items = api_method(_items)
                _east, max_width = self.get_table_column_width(
                                            items, max_width)
            east_asian_width.append(_east)
            results.append(items)

        # format table block considering its width
        lines = []
        fmt = u"  ".join(u"{%s:{%s}}" % (num, num + 1)
                         for num in range(0, len(max_width) * 2, 2))
        fmt = u"{0}{1}\n".format(indent, fmt)
        for rnum, row in enumerate(results):
            _width = list(max_width)  # copy
            len_width = map(len, row)
            for cnum, col in enumerate(row):
                if re.search(r"=+", col, re.U):
                    row[cnum] = col[0] * max_width[cnum]
                else:
                    _width[cnum] = max_width[cnum] - (
                        east_asian_width[rnum][cnum] - len_width[cnum])
            lines.append(fmt.format(*zip_with_flatlist(row, _width)))
        return api, lines

    def _call_for_tableblock(self, api_method, block_lines):
        def _get_column_length(line):
            table_type, length = None, 0
            match = re.search(_TABLEBLOCK, line)
            if match:
                d = match.groupdict()
                if d.get("grid_rule"):
                    table_type = "grid"
                    length = len(line.split("+"))
                elif d.get("simple_rule"):
                    table_type = "simple"
                    length = len(line.split())
            return table_type, length

        ret = [None, u""]
        table_type, col_len = _get_column_length(block_lines[0])
        max_width = [0] * col_len
        if table_type == "grid":
            ret = self._call_for_gridtable(api_method, block_lines, col_len)
        elif table_type == "simple":
            ret = self._call_for_simpletable(api_method, block_lines, col_len)
        return ret

    def _call_for_section(self, api_method, block_lines):
        api, results, width, indent = None, [], 0, ""
        for line in block_lines:
            if not re.search(_EMPTY_LINE, line) and \
               not re.search(_SECTION_LINE, line):
                match = re.search(_LINE_WITH_INDENT, line)
                api, line = self._call_keeping_prefix(api_method, line, match)
                width = get_east_asian_width(line) - 1  # -1 is linebreak
                if match:
                    indent, _ = match.groups()
                    width += len(indent)
            results.append(line)

        lines = []
        for num, line in enumerate(results):
            if re.search(r'[%s]{2,}\s+' % string.punctuation, line):
                line = u"{0}\n".format(line[0] * width)
            lines.append(line)
        return api, lines

    def _call_for_paragraph(self, api_method, block_lines):
        api, indents, lines = None, [], []
        for line in concatenate_lines(block_lines, _LINE_WITH_INDENT_OR_NONE):
            match = re.search(_LINE_WITH_INDENT, line)
            indent, line = self._markup_notranslate(line, match)
            indents.append(indent)
            lines.append(line)
        api, trans = self._call_keeping_prefix_with_array(api_method,
                                                          indents, lines)
        return api, self.split_lines_with_eos(trans)

    def call(self, api_method):
        for btype, block_lines, first in self.blocks:
            if btype == REST_BLOCK_TYPE["directive"]:
                r = self._call_for_directive(api_method, block_lines, first)
            elif btype == REST_BLOCK_TYPE["source"]:
                r = self._call_for_sourceblock(api_method, block_lines, first)
            elif btype == REST_BLOCK_TYPE["lineblock"]:
                r = self._call_for_lineblock(api_method, block_lines)
            elif btype == REST_BLOCK_TYPE["listblock"]:
                r = self._call_for_listblock(api_method, block_lines)
            elif btype == REST_BLOCK_TYPE["tableblock"]:
                r = self._call_for_tableblock(api_method, block_lines)
            elif btype == REST_BLOCK_TYPE["section"]:
                r = self._call_for_section(api_method, block_lines)
            elif btype == REST_BLOCK_TYPE["paragraph"]:
                r = self._call_for_paragraph(api_method, block_lines)
            else:
                r = (None, block_lines)
            lines = r[1]
            yield lines

    @classmethod
    def get_table_column_width(self, items, max_width, initial=None):
        width = [get_east_asian_width(text) for text in items]
        return width, [max(i) for i in map(initial, width, max_width)]

    @classmethod
    def get_indent_and_text(self, line):
        indent, text = "", line
        match = re.search(_LINE_WITH_INDENT, line)
        if match:
            indent, text = match.groups()
        return indent, text

    @classmethod
    def markup_paragraph_notranslate(self, text):
        text = text.replace(u"&", u"&amp;")
        text = text.replace(u"<", u"&lt;")
        text = text.replace(u">", u"&gt;")
        repl = r"<span class=notranslate>\1</span>"
        return re.sub(_NOTRANSLATE_PTRN, repl, text)



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        [u".. note::",
         u"",
         u"  first line",
         u"  second line",
         u""],
        [("d",
          [u".. note::",
           u"",
           u"  first line",
           u"  second line",
           u""],
          u".. note::"),
         4]
    ),
    (
        [u".. note::",
         u"  first line",
         u"  second line",
         u""],
        [("d",
          [u".. note::",
           u"  first line",
           u"  second line",
           u""],
          u".. note::"),
         3]
    ),
    (
        [u".. note::",
         u"  first line",
         u"  second line"],
        [("d",
          [u".. note::",
           u"  first line",
           u"  second line"],
          u".. note::"),
         2]
    ),
    (
        [u".. note::   ",
         u"  first line"],
        [("d",
          [u".. note::   ",
           u"  first line"],
          u".. note::   "),
         1]
    ),
    (
        [u".. image:: first.png",
         u".. image:: second.png",
         u""],
        [("d",
          [u".. image:: first.png"],
          u".. image:: first.png"),
         0]
    ),
    (
        [u".. image:: first.png",
         u"",
         u".. image:: second.png",
         u""],
        [("d",
          [u".. image:: first.png",
           u""],
          u".. image:: first.png"),
         1]
    ),
]



# -*- coding: utf-8 -*-

import abc
import re
import utils
from collections import defaultdict

from google import GoogleTranslator
from microsoft import MicrosoftTranslator
from ikazuchi.conf import show_how_to_get_apikey

__all__ = [
    "TRANSLATE_API",
]

class BaseTranslator(object):
    """Base class for Translator"""

    __metaclass__ = abc.ABCMeta
    api = lambda klass: klass.__class__.__name__.replace("Translating", "")
    # FIXME: span tag only, cannot match for minimal when html tag is nested
    notrans_tag = re.compile(
        r"<span class=[\"']?notranslate[\"']?>(.*?)</span>", re.IGNORECASE)
    whitespaces = re.compile(r"\s+", re.UNICODE)
    zerowidth = re.compile(u"\u200b|\u200c|\u200d|\ufeff", re.UNICODE)
    colon = re.compile(u"\s+(:+)$", re.UNICODE)

    @abc.abstractmethod
    def __init__(self, lang_from, lang_to, handler):
        """Overridden by MixIn class
        must be set "self.handler = handler"
        """

    @abc.abstractmethod
    def translate(self, text):
        pass

    def call_method_with_handler(self):
        method = getattr(self, self.handler.method_name)
        self.handler._call_method(method)

    def parse_html(self, html):
        from StringIO import StringIO
        from formatter import (AbstractFormatter, DumbWriter)
        from htmllib import HTMLParser
        _html = re.sub(self.notrans_tag, r" \1 ", html)
        buf = StringIO()
        p = HTMLParser(AbstractFormatter(DumbWriter(buf)))
        p.feed(_html)
        _sub = re.sub(self.whitespaces, " ", buf.getvalue())
        # FIXME: how can zerowidth be removed more simply?
        _sub = re.sub(self.zerowidth, "", _sub)
        _sub = re.sub(self.colon, r"\1", _sub)
        return _sub

    def set_apikey(self, key):
        self.apikey = key

    def set_apikey_from_conf(self, conf):
        _key = conf.get(self.api().lower(), "apikey")
        if _key:
            self.apikey = _key
        else:
            show_how_to_get_apikey()

    def set_parameter_from_conf(self, conf):
        import os
        from functools import partial
        general = partial(conf.get, "general")
        # urllib2.build_opener() skips ProxyHandler for Python 2.6/2.7
        # see also: http://bugs.python.org/issue7152#msg94150
        # so, the proxy settings set environment variable as workaround
        if general("http_proxy"):
            os.environ["http_proxy"] = general("http_proxy")
        if general("https_proxy"):
            os.environ["https_proxy"] = general("https_proxy")
        self.set_apikey_from_conf(conf)

# MixIn each implemented Translator
class TranslatingGoogle(GoogleTranslator, BaseTranslator):
    pass

class TranslatingMicrosoft(MicrosoftTranslator, BaseTranslator):
    pass


TRANSLATE_API = defaultdict(lambda: TranslatingMicrosoft,
    {
        "google": TranslatingGoogle,
        "microsoft": TranslatingMicrosoft,
    }
)



# -*- coding: utf-8 -*-

DATA_SET = [
    (   # inline markup 1
        u"rest *inline* **markup** is very ``simple``.",

        u"rest <span class=notranslate>*inline*</span> "\
        u"<span class=notranslate>**markup**</span> is very "\
        u"<span class=notranslate>``simple``</span>."
    ),
    (   # inline markup 2
        u"Now, `ikazuchi` is under development.",

        u"Now, <span class=notranslate>`ikazuchi`</span> "\
        u"is under development."
    ),
    (   # inline markup 3
        u"The relation ``Translator`` and ``Handler``.",

        u"The relation <span class=notranslate>``Translator``</span> and "\
        u"<span class=notranslate>``Handler``</span>."
    ),
    (   # hyper link 1
        u"a link `ikazuchi <http://pypi.python.org/pypi/ikazuchi/>`_",

        u"a link <span class=notranslate>`ikazuchi &lt;"\
        u"http://pypi.python.org/pypi/ikazuchi/&gt;`_</span>",
    ),
    (   # hyper link 2
        u".. _ikazuchi: http://pypi.python.org/pypi/ikazuchi/",

        u"<span class=notranslate>.. _ikazuchi: "\
        u"http://pypi.python.org/pypi/ikazuchi/</span>"
    ),
    (   # hyper link 3
        u"unwrapped link http://pypi.python.org/pypi/ikazuchi/",

        u"unwrapped link <span class=notranslate>"\
        u"http://pypi.python.org/pypi/ikazuchi/</span>"
    ),
    (   # hyper link 4
        u"a link name `ikazuchi`__",

        u"a link name <span class=notranslate>`ikazuchi`__</span>"
    ),
    (   # hyper link 5
        u"__ http://pypi.python.org/pypi/ikazuchi/",

        u"<span class=notranslate>__ "\
        u"http://pypi.python.org/pypi/ikazuchi/</span>"
    ),
    (   # text containing '&'
        u"$ chrome &",

        u"$ chrome &amp;"
    ),
    (   # role
        u"it can :role:`name`, for example :mod:`ikazuchi`.",

        u"it can <span class=notranslate>:role:`name`</span>, "\
        u"for example <span class=notranslate>:mod:`ikazuchi`</span>."
    ),
    (   # rubric 1
        u"ikazuchi [#f1]_ is a tool.",

        u"ikazuchi <span class=notranslate>[#f1]_</span> is a tool."
    ),
    (   # rubric 2
        u".. [#f1] ikazuchi's document",

        u"<span class=notranslate>.. [#f1] ikazuchi's document</span>"
    ),
    (   # reference 1
        u"ikazuchi [ref]_ is a tool.",

        u"ikazuchi <span class=notranslate>[ref]_</span> is a tool."
    ),
    (   # reference 2
        u".. [ref] ikazuchi's document",

        u"<span class=notranslate>.. [ref] ikazuchi's document</span>"
    ),
    (   # source 1
        u"description: that is, ",

        u"description<span class=notranslate>: </span>that is, "
    ),
    (   # source 2
        u"::\n  source\n  block",

        u"<span class=notranslate>::\n  </span>source\n  block"
    ),
]



# -*- coding: utf-8 -*-

try:
    import vim
except ImportError:
    print "call ':pyfile enrai' from vim"

from ikazuchi.core.translator import TRANSLATE_API
from ikazuchi.vim.utils import (to_unicode, get_vim_variables,
                                get_apikey, get_translate_method)

def get_word_on_cursor(enc):
    words = to_unicode(vim.current.line.split(" "), enc)
    cur_row, cur_column = vim.current.window.cursor
    cur_column += 1  # add 1 since cursor column start from 0
    word_pos = 0
    for word in words:
        word_pos += len(word) + 1  # for whitespace
        if cur_column <= word_pos:
            return word
    return None

def translate_with_word(api_method, enc):
    word = get_word_on_cursor(enc)
    if word:
        api, trans = api_method([word])
        vim.command("let enrai_target_word='{0}'".format(word.encode(enc)))
        print "Translated by {0}: {1}".format(api, trans[0].encode(enc))

def translate(api_name, lang_from, lang_to, enc):
    t = TRANSLATE_API[api_name](lang_from, lang_to, None)
    t.set_apikey(get_apikey(api_name))
    api_method = get_translate_method(t, api_name)
    return translate_with_word(api_method, enc)

def main():
    try:
        vim_vars = get_vim_variables()
        translate(*vim_vars)
    except Exception as err:
        print "Got error: {0}".format(err)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import sys
from os.path import abspath, dirname

import pep8

CURRENT_DIR = dirname(abspath(__file__))
BASE_DIR = dirname(CURRENT_DIR)


def test_pep8():
    arglist = [
        ['statistics', True],
        ['show-source', True],
        ['repeat', True],
        ['exclude', ['gen-py', 'GAuth.py', 'ttypes.py', 'constants.py']],
        ['ignore', [
            'E126', 'E127', 'E128', 'E302', 'E701', 'E731',
        ]],
        ['paths', [BASE_DIR]],
    ]

    pep8style = pep8.StyleGuide(arglist, parse_argv=False, config_file=False)
    options = pep8style.options
    if options.doctest:
        import doctest
        fail_d, done_d = doctest.testmod(report=False, verbose=options.verbose)
        fail_s, done_s = selftest(options)
        count_failed = fail_s + fail_d
        if not options.quiet:
            count_passed = done_d + done_s - count_failed
            print("%d passed and %d failed." % (count_passed, count_failed))
            print("Test failed." if count_failed else "Test passed.")
        if count_failed:
            sys.exit(1)
    if options.testsuite:
        init_tests(pep8style)
    report = pep8style.check_files()
    if options.statistics:
        report.print_statistics()
    if options.benchmark:
        report.print_benchmark()
    if options.testsuite and not options.quiet:
        report.print_results()
    if report.total_errors:
        if options.count:
            sys.stderr.write(str(report.total_errors) + '\n')
        sys.exit(1)

    # reporting errors (additional summary)
    errors = report.get_count('E')
    warnings = report.get_count('W')
    message = 'pep8: %d errors / %d warnings' % (errors, warnings)
    print message
    assert report.total_errors == 0, message



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        u"text line",
        (u"", u"text line")
    ),
    (
        u"  ",
        (u" ", u" ")
    ),
    (
        u" \t ",
        (u" \t", u" ")
    ),
    (
        u"  text line",
        (u"  ", u"text line")
    ),
    (
        u"\ttext line",
        (u"\t", u"text line")
    ),
    (
        u"\t  text line",
        (u"\t  ", u"text line")
    ),
    (
        u"\t  \ntext line",
        (u"\t  \n", u"text line")
    ),
]



# -*- coding: utf-8 -*-

import sys
from setuptools import setup, find_packages
from os.path import join as pathjoin

VERSION = "0.5.6"
LONG_DESCRIPTION = "".join([
    open(pathjoin("src","README.txt")).read(),
    open(pathjoin("src","TODO.txt")).read()])

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Environment :: Web Environment",
    "Intended Audience :: Developers",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: Apache Software License",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: POSIX",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 2 :: Only",
    "Topic :: Software Development",
    "Topic :: Software Development :: Internationalization",
]

INSTALL_REQUIRES = ["setuptools"]
if sys.version_info < (2, 7):
    INSTALL_REQUIRES.append("argparse")

setup(
     name="ikazuchi",
     version=VERSION,
     description="ikazuchi helps to translate document using web translate APIs efficiently",
     long_description=LONG_DESCRIPTION,
     classifiers=CLASSIFIERS,
     keywords=["translate", "i18n", "internationalization"],
     author="Tetsuya Morimoto",
     author_email="tetsuya.morimoto@gmail.com",
     url="http://t2y.bitbucket.org/ikazuchi/build/html/index.html",
     license="Apache License 2.0",
     py_modules=[],
     scripts=["src/ikazuchi/vim/raimei", "src/ikazuchi/vim/enrai"],
     packages=find_packages("src"),
     package_dir={"": "src"},
     package_data={"": ["buildout.cfg"]},
     namespace_packages=["ikazuchi", "ikazuchi.plugins"],
     include_package_data=True,
     install_requires=INSTALL_REQUIRES,
     extras_require=dict(
         test=[
             "Nose",
             "minimock",
             "pep8",
         ],
     ),
     test_suite="nose.collector",
     tests_require=["Nose","minimock","pep8"],
     entry_points={
        "console_scripts": [
            "ikazuchi = ikazuchi.ikazuchi:main",
        ],
        "ikazuchi": [
            "ikazuchi = ikazuchi.ikazuchi",
            "raimei = ikazuchi.vim.raimei:main",
            "enrai = ikazuchi.vim.enrai:main",
        ],
        "ikazuchi.plugins": [
            "rstfile = ikazuchi.plugins.rstfile",
        ],
     }
)



# -*- coding: utf-8 -*-

from nose.tools import *

# functions for test
from ikazuchi.core.translator import TranslatingMicrosoft

class TestMicrosoftTranslator(object):

    def setup(self):
        self.t = TranslatingMicrosoft("ja", "en", None)

    def test_get_api_path(self):
        def break_sentences():
            pass

        def get_language_names():
            pass

        assert_equal("BreakSentences?", self.t.get_api_path(break_sentences))
        assert_equal("GetLanguageNames?",
                     self.t.get_api_path(get_language_names))

    def test_break_sentences(self):
        assert_equal(('Microsoft', ['10', '5']),
                     self.t.break_sentences("ikazuchi. test."))

    def test_detect(self):
        assert_equal(('Microsoft', 'ja'),
                     self.t.detect(unicode("テスト", "utf-8")))

    def test_detect_array(self):
        texts = [
            u"ikazuchi",
            u"bonjour",
            unicode("テスト", "utf-8"),
            unicode("你好", "utf-8"),
        ]
        assert_equal(('Microsoft', ['en', 'fr', 'ja', 'zh-CHS']),
                     self.t.detect_array(texts))

    def test_get_language_names(self):
        lang_codes = ["en", "fr", "ja", "zh-CN"]
        assert_equal(('Microsoft', ['English', 'French', 'Japanese',
                        "Chinese Simplified (People's Republic of China)"]),
                     self.t.get_language_names(lang_codes))

    def test_get_languages_for_speak(self):
        assert_equal(
            ('Microsoft', ['ar', 'ar-eg', 'ca', 'ca-es', 'da', 'da-dk', 'de',
                'de-de', 'en', 'en-au', 'en-ca', 'en-gb', 'en-in', 'en-us',
                'es', 'es-es', 'es-mx', 'fi', 'fi-fi', 'fr', 'fr-ca', 'fr-fr',
                'hi', 'hi-in', 'it', 'it-it', 'ja', 'ja-jp', 'ko', 'ko-kr',
                'nb-no', 'nl', 'nl-nl', 'no', 'pl', 'pl-pl', 'pt', 'pt-br',
                'pt-pt', 'ru', 'ru-ru', 'sv', 'sv-se', 'yue', 'zh-chs',
                'zh-cht', 'zh-cn', 'zh-hk', 'zh-tw']),
            self.t.get_languages_for_speak())

    def test_get_languages_for_translate(self):
        assert_equal(
            ('Microsoft',
                ['af', 'ar', 'bs-Latn', 'bg', 'ca', 'zh-CHS', 'zh-CHT', 'yue',
                 'hr', 'cs', 'da', 'nl', 'en', 'et', 'fj', 'fil', 'fi', 'fr',
                 'de', 'el', 'ht', 'he', 'hi', 'mww', 'hu', 'id', 'it', 'ja',
                 'sw', 'tlh', 'tlh-Qaak', 'ko', 'lv', 'lt', 'mg', 'ms', 'mt',
                 'yua', 'no', 'otq', 'fa', 'pl', 'pt', 'ro', 'ru', 'sm',
                 'sr-Cyrl', 'sr-Latn', 'sk', 'sl', 'es', 'sv', 'ty', 'th',
                 'to', 'tr', 'uk', 'ur', 'vi', 'cy']),
            self.t.get_languages_for_translate())

    def test_translate(self):
        assert_equal(('Microsoft', 'Test'),
                     self.t.translate(unicode("テスト", "utf-8")))

    def test_translate_array(self):
        texts = [
            unicode("テスト", "utf-8"),
            unicode("ペンギン", "utf-8"),
            unicode("鉛筆", "utf-8"),
        ]
        assert_equal(('Microsoft', ['Test', 'Penguin', 'Pencil']),
                     self.t.translate_array(texts))



# -*- coding: utf-8 -*-

import abc

class BaseHandler(object):
    """Base class for handler"""

    __metaclass__ = abc.ABCMeta

    method_name = "translate"

    @abc.abstractmethod
    def _call_method(self, method):
        pass

class NullHandler(BaseHandler):
    """Handler class for Null object"""
    def _call_method(self, method):
        pass  # having nothing to do

class LanguageHandler(BaseHandler):
    """Handler class for languages API"""
    def __init__(self, api, encoding):
        self.api = api
        self.encoding = encoding
        if self.api == "google":
            self.method_name = "languages"
        elif self.api == "microsoft":
            self.method_name = "get_languages_for_translate"
        else:
            raise NotImplementedError("Not Supported")

    def _encode(self, text):
        return text.encode(self.encoding[1])

    def _call_method(self, api_method):
        api, result = api_method()
        for r in result:
            _method = u"{0}({1}):".format(self.method_name, api)
            if self.api == "google":
                print self._encode(u"{0:25}{language:5} {name}".format(
                                   _method, **r))
            else:
                print self._encode(u"{0:25} {1}".format(_method, r))



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        [u"something::",
         u"",
         u"  first code",
         u"  second code",
         u""],
        [("so",
          [u"something::",
           u"",
           u"  first code",
           u"  second code",
           u""],
          u"something::"),
         4]
    ),
    (
        [u"something::",
         u"  first code",
         u"  second code",
         u""],
        [("so",
          [u"something::",
           u"  first code",
           u"  second code",
           u""],
          u"something::"),
         3]
    ),
    (
        [u"something::",
         u"",
         u"  first code",
         u"  second code"],
        [("so",
          [u"something::",
           u"",
           u"  first code",
           u"  second code"],
          u"something::"),
         3]
    ),
    (
        [u"something::",
         u"  first code",
         u"  second code"],
        [("so",
          [u"something::",
           u"  first code",
           u"  second code"],
          u"something::"),
         2]
    ),
    (
        [u"::",
         u"  first code"],
        [("so",
          [u"::",
           u"  first code"],
          u"::"),
         1]
    ),
    (
        [u"::  ",
         u"  first code"],
        [("so",
          [u"::  ",
           u"  first code"],
          u"::  "),
         1]
    ),
]
